import numpy as np
import pylab as pl



def add_pattern_frames(t_min=0.0, t_max=1.0, dt=0.001, ctemplate=None, corner=None, color=None, ax=None):
    pl.interactive(False)
    if ax is None:
        fig = pl.figure(figsize=(12,6))
        ax = fig.add_axes((0.1,0.1,0.85,0.85))
    assert ctemplate.shape[1] == len(corner)
    assert ctemplate.shape[1] == len(color)
    # save aspect
    xlim = ax.get_xlim()
    ylim = ax.get_ylim()
    # get frames
    K = ctemplate.shape[1]
    tn_min = int(round(t_min/dt))
    tn_max = int(round(t_max/dt))
    ctemplate = ctemplate[tn_min:tn_max]
    tn_to_t = lambda i: (i + tn_min) * dt
    frame = []      # [(k, t_on, t_off), ...)
    latest_on = np.zeros(K, dtype=int)
    onoff = ctemplate[0:-1]-ctemplate[1:]   # -1 = on, 1 = off
    for tn,k in zip(*onoff.nonzero()):
        if onoff[tn,k] == -1:
            latest_on[k] = tn
        if onoff[tn,k] == +1:
            frame.append((k, tn_to_t(latest_on[k]+1), tn_to_t(tn+1)))
            latest_on[k] = -1
    for k in (latest_on > 0).nonzero()[0]:   # collect missing offs
        frame.append((k, tn_to_t(latest_on[k]+1), tn_to_t(tn_max-tn_min)))
    # plot frames
    #print frame
    for k, t_on, t_off in frame:
        for ymin, ymax in corner[k]:
            #print "   > Add rect l,b,w,t = ", (t_on, ymin, t_off-t_on, ymax-ymin)
            patch = pl.Rectangle((t_on, ymin), t_off-t_on, ymax-ymin, ec=color[k], lw=0.5, fill=False, alpha=0.5)
            ax.add_patch(patch)
    ax.set_xlim(*xlim)
    ax.set_ylim(*ylim)
    pl.draw()
    pl.interactive(True)
    return ax



def calc_posterior(y, W, V, b, Ak):
    exp = np.exp
    dot = np.dot
    K = W.shape[0]
    Z = np.transpose(np.unravel_index(np.arange(2**K), [2]*K))
    P = exp(0.5 * dot(Z, dot(W, Z.T)).diagonal() + dot(Z, dot(V, y)) + dot(Z,b-Ak))
    P /= P.sum()
    return P

def marginals(P):
    K = np.log2(len(P)).astype(int)
    Z = np.transpose(np.unravel_index(np.arange(2**K), [2]*K))
    p = np.zeros(K)
    for k in xrange(K):
        idx = Z[:,k] == 1
        p[k] = P[idx].sum()
    return p













