import numpy as np
import time
import sys
sys.path.append('../')
import gwta_shared as gwta
import time
from billtool import generate_DSL

# # # # # # # # # # # # # # # # # # # # # # # #
# # # G E N E R A T I N G   P A R A M S # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

np.random.seed(0)       # seed for parameter generation
DSL = "Fig2_eval"                # Data Set Locator defines the directory name where
                            # results are stored within the "out/"-directory
#DSL = generate_DSL()        # Use this to generate a timestamp-DSL
#                            # If used, the DSL has to be changed in other scripts

T_si = 2.5 #30.0                   # Total time in SI units
dt = 0.001                   # discretization time step in SI units
tau_si = 0.010               # tau in SI units

K = 6*1 # 20**2 - 20
N = 3*6*6 # 20**2
dims = (18,6) # (20,20)
connradius_V = 2.6 # 2.6
num_subnet = 1
num_parallel = 1000

template = np.load("data/template_2500x6x108_uni_v2.npy")
r = np.load("data/rate_6x36_uniform.npy")
#template = template.reshape(template.shape[0], -1)
nullcause = template == 0.
template = 0.2 + (0.55 - 0.2) * template
template[nullcause] += 0.

piki = np.zeros((K,N))
left = lambda k:  (6*(-1 + 3 * k)) % N
for k,p in enumerate(r):
    piki[k,:36] = p
    piki[k] = np.roll(piki[k], left(k))

piki = 0.2 + (0.55 - 0.2) * piki

# # # # # # # # # # # # # # # # # # # # # # # #
# # # # # P R E P R O C E S S I N G # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

tau = int(round(tau_si/dt))
T = int(round(T_si/dt))



# # # # # # # # # # # # # # # # # # # # # # # #
# # # # # A C T U A L   P A R A M S # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

param = {
    "global" : dict(
        T_si = T_si,
        T = T,
        dt = dt,
        seed = 1,
        DSL = DSL,
        num_parallel = num_parallel,           # level of parallelization during learning (int or 'auto')
        spiking = True,             # whether to use spikes streams or independent samples
        Tplast = (0, T),            # plasticity will be switched on/off at these times 
        TEST = False,                # whether to save params and results
        PLOTEVERY = 1000,           # Plotter update, timesteps or None/0
        SAVEEVERY = 1000,            # save only every n-th step (for logger)
         ),
    "input" : dict(
         NOINPUT = False,            # present zeros only
         SAMEINPUT = True,
         InputClass = gwta.FromTemplateBernoulliGenerator,
         input_sequence = np.repeat(np.arange(T), num_parallel).reshape(T, num_parallel), # None or array of idx for template[idx]
         inputKwargs = {"template" : template, "duration" : int(round(0.500/dt))},
         pspKwargs = {'kernel' : [1.]*tau, 'postprocess' : lambda y : np.clip(y, 0, 1)},    # rectangular, renewing PSPs
         ),
    "net" : dict(
        dims = dims,
        K = K,
        N = N,
        tau = tau,
        kwargs_poszk = {"dist" : (3,100), "offset" : (1.5,1.5)},
        kwargs_posyi = {"dist" : (1,1), "offset" : (0,0)},
        kwargs_connV = {"connradius" : connradius_V, "norm" : "max-norm", "p_plast" : 1.0, "Vinit" : 0.0},
        kwargs_connW = {"connradius" : 100., "num_net" : num_subnet, "w" : 0., 
                        "norm" : "max-norm", "b_init" : -1., "W_inh" : -100.},
        VsynapseClass = gwta.BernoulliSynapses,
        VKwargs = {'syninit' : lambda k,i: 0., 'pi0i' : np.ones(N) * 0.2,
                                                            #np.ones(template.shape[1]) * template.min(),
                                                            # template.mean(0),
                                                            # np.ones(template.shape[1]) * template.min(),
                   'semlike' : False, 'calcAk' : True, 'only_exc' : True},
        setV0_inpmean = False,
        set_piki_from_array = piki,  # False or array of shape (K,N)
        load_b_from_file = False,  # False or filename
        load_V_from_file = False,  # False or filename
        load_W_from_file = "data/W.npy",  # False or filename
        samplerClass =  gwta.SamplerAbsref,
        samplerKwargs =  {'tau': tau, 'shuffleIdx': True },
        m_hom = 0.277, # 0.3 * 4 / (2*connradius_V)**2 # 0.5 * 4. / K, # 0.1  / (2*connradius_V+1)**2,
        ),
    "train" : dict(
        eta_V = 0.0,
        eta_b = 0.0,
        eta_W = 0.0,
        rec_plast_rule = gwta.covariance_plasticity_tan,
        only_exc_W = True,
        eta_scale = lambda t: 2. * 1./tau * ( (0.01 - 1.)/float(T) * t + 1. ), # 1.0 * 1./tau, # 
        ),
}




