import numpy as np
import pylab as pl
import sys
sys.path.append('../')
import my_matplotlib_defaults
from figure_format_plos import fig_width, cm2inch
from func import *
from gwta_shared import color as gc
from gwta_shared import plot_colored_styletrain

#from gwta.builder import dist_pbc


# # # # # # # # # # # # 
# # # P A R A M S # # # 
# # # # # # # # # # # # 

ZOOM = 2.0
SUBPLOTS = ("sketch", "weight", "spike", "posterior", ) 
SAVE = True

DSL = "Fig2_eval" 


files = dict(ZS=("spike",),    # Files required by different subplots: DSL_KEY.npy=("A", "B")
             Z=("posterior",),
             YS=("spike",),
             Y=("posterior", "weight"),
             V=("weight", "posterior",),
             V0 = ("weight",),
             W=("posterior",),
             Ak=("posterior",),
             b=("posterior",),
             V_mask=("weight",),
             dims=("spike",),
             centers=("weight",),
            )

# For 6x6 rec field
preidx = np.array([np.arange(-6+18*k, -6+18*k+36) % 108 for k in xrange(6)])
corner = [((-10,29.3), (101.7,500),), 
            ((11.7,47.3),), 
            ((29.7,65.3),), 
            ((47.7,83.3),), 
            ((65.7,101.3),), 
            ((83.7,500), (-10,11.3)),]


param_posterior = dict(kernel=[1./20]*20, highlight=(2,3))

colorseq = (gc.select['blue'], gc.select['green'], gc.select['red'], 
            gc.select['cyan'], gc.select['yellow'], gc.select['magenta'])

param_spike = dict(
    trial = 7,
    t_min = 2.1 - 1.5, # 1.08
    t_max = 2.1,
    dt = 0.001,
    color = colorseq,
    preidx = preidx,
    corner = corner,
    pattern = np.load("data/pattern_2500x6_v2.npy"),
    bgcolor_z = gc.network_spikebg,
    bgcolor_y = gc.input_spikebg,
    )



param_weight = dict(k=2, rfdims=(6,6), 
                    vmin=0.1, vmax=0.6, #0.7
                    v_of_pi = lambda pi: np.log(pi/(1.-pi)) - V0[0],
                    inpidx=preidx, t_slice=slice(1210,1510,1))


# in figure coords; orientation up = 0.
param_stars = dict(xy=([0.571,0.634],[0.805,0.634]),)
param_lines = dict(rate_hori_xy=([[0.720,0.219], [0.919,0.219]], [1,1,1,1,1,1,1,1,1,1]),
                   weight_hori_xy=([[0.874,0.532], [0.919,0.532]], [1,1,1,1,1,1,1,1,1,1]),
                   )



# # # # # # # # # # #
# # # S E T U P # # # 
# # # # # # # # # # # 

print " > DSL =", DSL
print " > Subplots:", SUBPLOTS

# Zooming
pl.rc("figure", dpi=ZOOM*pl.rcParams['figure.dpi'])

# Figure setup
fig = pl.figure(figsize=(fig_width['two_col'], cm2inch(8.0)))
axes = dict()

top = 0.96
lpad, rpad = 0.04, 0.025
wfig = 1. - lpad - rpad
r = fig.get_figheight() / fig.get_figwidth()

# add axes
# sketches
w = 0.155*wfig
h = w / r
print " > Axis 'graph' size in inch: %.3f x %.3f" % (w * fig.get_figwidth(), h * fig.get_figheight())
rect = (lpad, top-h-0.015, w, h)
axes['graph'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal', frame_on=True)
rect = (lpad + w + 0.04, top-h-0.02, w, h)
print " > Axis 'neural' size in inch: %.3f x %.3f" % (w * fig.get_figwidth(), h * fig.get_figheight())
axes['neural'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal', frame_on=True)

w = 2 * w + 0.04
h = 5./8. * w / r
rect = (lpad, 0.09, w, h)
axes['network'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=True)
print " > Axis 'network' size in inch: %.3f x %.3f" % (w * fig.get_figwidth(), h * fig.get_figheight())

# weight / rate
top = 0.532 + 0.06
w = 0.055
h = w / r
rect = (1.-rpad-w, top-h, w, h)
axes['weight'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal', frame_on=True)
rect = (1.-rpad-w, top-h-0.03, w, 0.016)
axes['weight_cb'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=True)

top = 0.278
rect = (1.-rpad-w, top-h, w, h)
axes['pik'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal', frame_on=True)
rect = (1.-rpad-w, top-h-0.03, w, 0.016)
axes['pik_cb'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=True)


## simulation
top = 0.9485
l = lpad + 0.47 * wfig
w = 0.42 * wfig
h=0.120
dh = (0.92 - 6.5*h) / 4.
rect = (l, top-h, w, h)
axes['pz'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=True)

rect = (l, top-2*h-dh, w, h)
axes['pz_sim'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=True)

rect = (l, top-3*h-3*dh, w, h)
axes['ZS'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=True)

rect = (l, 0.03, w, 3.5*h)
axes['YS'] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=True)




# Overlay
rect = (0.0002, 0.0002, 0.9996, 0.9996)
axes["all"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=False, zorder=0)
letters = dict(A=(0.015,0.96), B=(0.205,0.96), D=(0.405,0.96),
               C=(0.015,0.535),                E=(0.405,0.605), F=(0.895,0.605))
for l,(x,y) in letters.iteritems():
    axes["all"].text(x, y, l, fontsize=8, fontweight='bold', ha='center', va='baseline')

# stars
ax = axes['all']
for x,y in param_stars['xy']:
    #l = 1.01
    #dx,dy = -np.sin(2*pi * phi / 360.) * l, np.cos(2*np.pi * phi / 360.) * l
    #patch = pl.Arrow(x-dx, y-dy, dx, dy, width=1., ec='w', fc='k', lw=1.)
    #ax.add_patch(patch)
    ax.text(x,y-0.01,'*', ha='center')

# lines
ax = axes['all']
for xy,dash in param_lines.itervalues():
    x,y = np.array(xy).T
    l, = ax.plot(x, y, 'k', alpha=0.8)
    if dash is not None:
        l.set_dashes(dash)

ax.set_xlim(0,1)
ax.set_ylim(0,1)
pl.draw()



# # # # # # # # # # # # # # # 
# # # L O A D   D A T A # # # 
# # # # # # # # # # # # # # # 

print " > Load data."
fname = lambda s: "out/"+DSL+"/"+s+".npy"
for key in files:
    if np.intersect1d(files[key], SUBPLOTS).size > 0:
        print "   > Attempt to load", key, "from", fname(key),
        try:
            globals()[key] = np.load(fname(key))
            print "[ok]."
        except Exception, e:
            print "[FAILED]."
            print "     >", repr(e)
    else:
        print "   >", key, "data not needed for selected subplots."


# post process
if "ZS" in globals():
    ZS = ZS[:,param_spike['trial']]    # one network only
    K = ZS.shape[-1]
if "Z" in globals():
    Z_mean = Z.mean(1)
    #Z = Z[:,0]    # one network only
    #K = Z.shape[-1]
if "YS" in globals():
    YS = YS[:,0]    # one network only
    N = YS.shape[-1]
if "Y" in globals():
    Y = Y[:,0]    # one network only
    N = Y.shape[-1]
if "YT" in globals():
    YT = YT[:,0]    # one network only



# # # # # # # # # # # # #
# # #   T O O L S   # # #
# # # # # # # # # # # # #



# # # # # # # # # # # #
# # #   P L O T   # # #
# # # # # # # # # # # #


if "sketch" in SUBPLOTS:
    print " > Render sketch."
    for ax,f in ([axes["graph"], "fig/fig2a.png"], 
                 [axes["neural"], "fig/fig2b.png"],
                 [axes["network"], "fig/fig2c.png"]):
        im = pl.imread(f)
        print "  > Image %s has dims" % f, im.shape[:2]
        ax.imshow(im, interpolation='lanczos')
        ax.set_frame_on(False)




if "weight" in SUBPLOTS:
    print " > Render weight."
    # synaptic weight
    ax = axes["weight"]
    k = param_weight['k']
    rfdims = param_weight['rfdims']
    v = V[k,V_mask[k]].reshape(rfdims)
    vmin, vmax = 0, np.ceil(10*v.max())/10 + 0.1 
    mappable = ax.imshow(v, cmap=gc.weight_cmap, origin='lower', vmin=vmin, vmax=vmax)
    weight_cb = pl.colorbar(mappable,  cax=axes['weight_cb'], orientation='horizontal')
    weight_cb.set_ticks(np.linspace(vmin, vmax, 2))
    weight_cb.ax.xaxis.set_tick_params('major', size=0)
    weight_cb.set_label("Weight V", labelpad=4)
    # input pattern
    ax = axes["pik"]
    v = Y[param_weight['t_slice'], param_weight['inpidx'][k]].mean(0).reshape(rfdims)
    vmin, vmax = param_weight['vmin'], param_weight['vmax']# np.ceil(10*v.max())/10
    mappable = ax.imshow(v, cmap=gc.input2_cmap, origin='lower', vmin=vmin, vmax=vmax)
    pik_cb = pl.colorbar(mappable,  cax=axes['pik_cb'], orientation='horizontal')
    pik_cb.set_ticks(np.linspace(vmin, vmax, 2))
    pik_cb.ax.xaxis.set_tick_params('major', size=0)
    pik_cb.set_label("Avg. input", labelpad=4)



if "posterior" in SUBPLOTS:
    print " > Render posterior."
    pl.interactive(False)
    ax = axes["pz"]
    tn_min, tn_max = param_spike['t_min'] / param_spike['dt'], param_spike['t_max'] / param_spike['dt']
    kernel = param_posterior['kernel']
    tn_min -= len(kernel)
    tn_max += len(kernel)
    p_theo = np.array([marginals(calc_posterior(y=Y[t], W=W, V=V, b=b, Ak=Ak)) for t in np.arange(tn_min,tn_max,dtype=int)])
    for k,(pk,c) in enumerate(zip(p_theo.T, param_spike['color'])):
        pk = np.convolve(kernel, pk, mode='same')[len(kernel):-len(kernel)]
        lw = 0.5
        if k in param_posterior['highlight']:
            lw *= 1
        ax.plot(pk, '-', lw=lw, c=c)
    ax.set_xlim(0, len(pk)-1)
    ax.set_ylim(-0.02,1.02)
    ax.set_yticks([0,1])
    ax.set_xticks([])
    ax.set_ylabel("Theory\np(z|y(t))", labelpad=10, ha='center')
    ax.yaxis.set_tick_params(size=1)
    ax.xaxis.set_tick_params(size=1)
    pl.draw()
    # Avg Network response
    ax = axes["pz_sim"]
    for k,(pk,c) in enumerate(zip(Z_mean[tn_min:tn_max].T, param_spike['color'])):
        pk = np.convolve(kernel, pk, mode='same')[len(kernel):-len(kernel)]
        lw = 0.5
        if k in param_posterior['highlight']:
            lw *= 1
        ax.plot(pk, '-', lw=lw, c=c)
    ax.set_xlim(0, len(pk)-1)
    ax.set_ylim(-0.02,1.02)
    ax.set_yticks([0,1])
    ax.set_ylabel("Network\np(z|y(t))", labelpad=10, ha='center')
    ax.yaxis.set_tick_params(size=1)
    ax.xaxis.set_tick_params(size=1)
    pl.draw()
    pl.interactive(True)

if "spike" in SUBPLOTS:
    print " > Render spikes."
    # Z SPIKES
    ax = axes["ZS"]
    kwargs = dict(dt=0.001, t_min=param_spike['t_min'], t_max=param_spike['t_max'],
                  ctemplate=ZS, cidx=np.arange(K).reshape(K,1), color=param_spike['color'],
                  default_color='k', tau=None, ms=3.0, marker='|', bgcolor=param_spike['bgcolor_z'],
                  show_scale=False)
    ax = plot_colored_styletrain(ZS, ax=ax, **kwargs)
    ax.set_ylabel("Network", labelpad=5)
    # Y SPIKES
    ax = axes["YS"]
    kwargs = dict(dt=0.001, t_min=param_spike['t_min'], t_max=param_spike['t_max'],
                  ctemplate=param_spike['pattern'], cidx=param_spike['preidx'], color=param_spike['color'],
                  default_color='0.5', tau=None, ms=1.0, bgcolor=param_spike['bgcolor_y'], show_scale=True)
    ax = plot_colored_styletrain(YS, ax=ax, alpha=0.7, **kwargs)
    ax.set_ylabel("Input neuron", labelpad=-5)
    # draw data highlighting rectangle
    p = param_weight
    dt = param_spike['dt']
    idx = p['inpidx'][p['k']]
    xy = (dt*p['t_slice'].start-0.004, min(idx)-0.8)
    w,h = dt*p['t_slice'].stop - dt*p['t_slice'].start + 0.008, max(idx) - min(idx) + 1.6
    patch = pl.Rectangle(xy, w, h, lw=0.0, ec='k', fc='k', alpha=0.2, zorder=0)
    ax.add_patch(patch)
    patch = pl.Rectangle(xy, w, h, lw=0.5, ec='k', fill=False, alpha=0.8, zorder=2)
    ax.add_patch(patch)
    pl.draw()



# # # # # # # # # # # # # #
# # # F I N A L I Z E # # #
# # # # # # # # # # # # # #


if SAVE is True:
    print " > Save figure to 'fig/Figure_wide.{png,pdf}'."
    fig.savefig("fig/Figure_wide.png")
    fig.savefig("fig/Figure_wide.pdf")

print " > Show figure."
pl.show()
print " > Done."






