import numpy as np
import pylab as pl
import time
from sys import stdout
import sys
sys.path.append('../')

import my_matplotlib_defaults
import gwta_shared as gwta

# # # INIT (extract and save params) # # #
paramfilename = gwta.get_param_file()
print " > Loading parameters from file " + paramfilename + ".py"
exec("from " + paramfilename + " import param")     # load parameter dictionary
gwta.extract_params(param, globals(), keep=('samplerKwargs', 'kwargs_poszk', 'kwargs_posyi', 'kwargs_connV', 'kwargs_connW', 'inputKwargs', 'pspKwargs', 'VKwargs'))

print " > DSL is", DSL
print " > Simulation will" + (spiking and ' ' or ' NOT ') + "be spiking."

# set random seed
np.random.seed(seed)

# create logger for sparsely saved data
kwargs = dict(Wtrace=("W", (K,K), 'float'), btrace=("b", (K,), 'float'), Vtrace=("V", (K,N), 'float'))
logger = gwta.Logger(DSL, T, SAVEEVERY, TEST, globals(), **kwargs)
logger.save_param(paramfilename)


# # # GENERATE NETWORK # # #

# Build network structure
net = gwta.NetworkBuilder(K, N, dims)
rdimsz = net.set_pos_raster("zk", **kwargs_poszk)
rdimsy = net.set_pos_raster("yi", **kwargs_posyi)
net.set_V(**kwargs_connV)
net.set_W(**kwargs_connW)
subnet = net.add_subnet(**kwargs_connW)
net.check()

# neuron and synapse arrays
V = VsynapseClass(K=K, N=N, mask=net.V_mask, **VKwargs)     # input synapse object
W = gwta.shmarray.zeros((K,K), dtype='d')                   # shared sampler matrix
b = gwta.shmarray.zeros(K, dtype='d')                       # shared sampler vector
V0 = V.get_V0()
ext = np.zeros((num_parallel,K))                            # vector for summed external input
V[:] = net.V
V_mask = net.V_mask
W[:] = net.W
W_mask = net.W_mask
b[:] = net.b

# load synapses and biases from file
if load_V_from_file:
    print " > Loading V from file '%s'." % load_V_from_file
    V.W[:] = np.load(load_V_from_file)

if load_b_from_file:
    print " > Loading b from file '%s'." % load_b_from_file
    b[:] = np.load(load_b_from_file)

if load_W_from_file:
    print " > Loading W from file '%s'." % load_W_from_file
    W[:] = np.load(load_W_from_file)

# init parallel processes
sampler = samplerClass(W, b, **samplerKwargs)
co = gwta.Coordinator(sampler, num_parallel, spiking)
num_parallel = co.num_parallel


# # # MEMORY CHECK # # #
# memory check for full array (Z, Y, shownTemplate, maybe: spikes)
tmpa = [((num_parallel,K),"int"),((num_parallel,N),"int"),((num_parallel,),"int")]
if spiking: tmpa += [((num_parallel,K),"int"), ((num_parallel,N),"int")]
gwta.memorycheck(T=T, arrays=tmpa, limit=0.8)


# # # GENERATE INPUT # # #
if NOINPUT:
    print " > WARNING: Input is switched off! Only zeros are presented!"
    try:
        template = inputKwargs['template']
    except:
        print " > Warning: No input templates defined."
    Y = np.zeros((T,num_parallel,N), dtype=np.float16)
    YT = Y
    if spiking:
        YS = Y
else:
    print " > Generating input of shape (%d,%d,%d)" % (T, num_parallel, N)
    try:
        template = inputKwargs['template']
    except:
        print " > Warning: No input templates defined."
    inpgen = InputClass(num_parallel=num_parallel, spiking=spiking, tau=tau, **inputKwargs)
    if spiking:
        #print "WARNING: HACK!"
        #seq = np.array(([7]*500 + [112]*500 + [71]*500)*40).reshape(60000,1)
        #YT, YS = inpgen.generate(T, sequence=seq)
        YT, YS = inpgen.generate(T, sequence=input_sequence)
        print " > Applying PSP kernel."
        Y = gwta.applyPSPkernel(S=YS, dtype=np.float16, **pspKwargs)
    else:
        YT, Y = inpgen.generate(T, sequence=input_sequence)

if SAMEINPUT:
    print " > Map same input to all samplers!"
    for n in xrange(1,num_parallel):
        Y[:,n] = Y[:,0]
        YT[:,n] = YT[:,0]
        if spiking:
            YS[:,n] = YS[:,0]


# Adapt V0
if setV0_inpmean:
    print " > Setting V0 to match input marginals."
    V.set_V0(V._logit(Y[:100000].mean(1).astype(float).mean(0)))
    V0 = V.get_V0()

if set_piki_from_array is not False:
    print " > Setting V from piki-array."
    piki = set_piki_from_array
    assert piki.shape == (K,N)
    V.set_centers(piki)
    if VKwargs['only_exc'] is True:
        np.clip(V.W, 0., 1000., out=V.W)



# # # RESULT ARRAYS # # #
Z = np.zeros((T,num_parallel,K), dtype=np.int8)
if spiking:
    ZS = np.zeros((T,num_parallel,K), dtype=np.int8)


# # # START SIM # # #
# start samplers
co.start_samplers()

# last preparations
if PLOTEVERY: plotter = gwta.LivePlotter(globals(), plotW=False)
ETA = lambda t: t and ( (float(T) / t - 1.) * (time.time() - time_start) )
reportmodulo = int(round(1./dt))
pe = gwta.ParameterEvolution(Nfilter=10, V=V.W, b=b, W=W)
print "\n > Start simulation of %.1fs (%d samples per sampler)" % (T_si, T)
time_start = time.time()


# # # MAIN LOOP # # #
for t in xrange(T):
    if t % reportmodulo == 0:
        print " > Time t=%ds \t ETA %.1fs" % (int(round(t*dt)), ETA(t))
        print "  > Mean(W) = %.2e" % (W[W_mask].mean())
        pe.get_stats(V=V.W, b=b, W=W)
    if PLOTEVERY and t % PLOTEVERY == 0:
        print " > plotting...",; stdout.flush()
        plotter.update()
        print "Done."; stdout.flush()
    # distribute input
    for i in xrange(num_parallel):
        ext[i] = V.transmit(Y[t,i])
    co.distribute_ext(ext)
    # unleash samplers
    co.sampling_initiate()
    # learning on previous states
    if Tplast[0] < t <= Tplast[1]:
        if eta_V > 0.:
            V.plasticity(eta_scale(t)*eta_V, Z[t-1], Y[t-1])
        if eta_b > 0.:
            b += eta_scale(t) * eta_b * num_parallel * (m_hom - Z[t-1].mean(0))
        if eta_W > 0.:
            W += eta_scale(t) * eta_W * W_mask * rec_plast_rule(W, Z[t-1], m_hom, alpha=1., exponent=2)
            if only_exc_W:
                W[:] = (1-W_mask) * W + W_mask * np.clip(W, 0., 10000.)
    # collect samples
    if spiking:
        Z[t], ZS[t] = co.sampling_collect()
    else:
        Z[t] = co.sampling_collect()
    # track results
    logger.track(t)


# # # FINALIZE AND SAVE RESULTS # # #
print " > Simulation took %.1fs" % (time.time() - time_start)
plotter.update()
co.finalize()

centers = V.get_centers()
Ak = V.get_Ak()

kwargs = dict(individual = ["subnet", "dims", "rdimsz", "rdimsy", "V0",
                            "V", "V_mask", "W", "W_mask", "b", "centers", "Ak", "Y", "YT", "YS", "Z", "ZS"],
              archive = ["Z", "Wtrace", "btrace", "Vtrace"])

if globals().has_key("template"):
    kwargs['individual'] += ["template"]

if spiking:
    kwargs['archive'] += ["ZS",]

logger.save(**kwargs)





