import numpy as np
import time
import sys
sys.path.append('../')
import gwta_shared as gwta
import time
from billtool import generate_DSL

# # # # # # # # # # # # # # # # # # # # # # # #
# # # G E N E R A T I N G   P A R A M S # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

np.random.seed(54321)       # seed for parameter generation
DSL = "Fig3_eval3"           # Data Set Locator defines the directory name where
                            # results are stored within the "out/"-directory
#DSL = generate_DSL()        # Use this to generate a timestamp-DSL
#                            # If used, the DSL has to be changed in other scripts

num_parallel = 1

T_si = 100. / num_parallel # Total time in SI units 25000
dt = 0.001                   # discretization time step in SI units
tau_si = 0.010               # tau in SI units

K = 7*3 # 20**2 - 20
N = 21*6 # 20**2
dims = (21,6) # (20,20)
connradius_V = 2.6
num_subnet = K

template = np.load("data/template-diri_1e5x7x3x126.npy")

template = 0.1 + (0.6 - 0.1) * template

## Original Paper Parameters
LOAD_TIME = 3000        # in (0, 1000, 3000, 10000)
## Quick Test Parameters
#LOAD_TIME = 100        # in (0, 1000, 3000, 10000)

LOAD_DSL = "Fig3_train"


# # # # # # # # # # # # # # # # # # # # # # # #
# # # # # P R E P R O C E S S I N G # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

tau = int(round(tau_si/dt))
T = int(round(T_si/dt))

load_fname = lambda s: "./out/"+LOAD_DSL+"/%s_t%d.npy" % (s,LOAD_TIME)

# Create and save snapshots of weights etc. at LOAD_TIME
x = np.load("out/"+LOAD_DSL+"/archive.npz")
vt = x['Vtrace']
v = vt[LOAD_TIME]
np.save(load_fname('V'),v)
bt = x['btrace']
b = bt[LOAD_TIME]
np.save(load_fname('b'),b)
Wt = x['Wtrace']
W = Wt[LOAD_TIME]
np.save(load_fname('W'),W)
Wm = np.load("out/"+LOAD_DSL+'/W_mask.npy')
np.save(load_fname('W_mask'),Wm)


# # # # # # # # # # # # # # # # # # # # # # # #
# # # # # A C T U A L   P A R A M S # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

param = {
    "global" : dict(
        T_si = T_si,
        T = T,
        dt = dt,
        seed = 53124,
        DSL = DSL,
        num_parallel = num_parallel,           # level of parallelization during learning (int or 'auto')
        spiking = True,             # whether to use spikes streams or independent samples
        Tplast = (0, T),            # plasticity will be switched on/off at these times 
        TEST = False,                # whether to save params and results
        PLOTEVERY = 10000,           # Plotter update, timesteps or None/0
        SAVEEVERY = 1000,            # save only every n-th step (for logger)
         ),
    "input" : dict(
         NOINPUT = False,            # present zeros only
         target_center = 0.1 + (0.6 - 0.1) * np.load("data/rate-diri03_7x3x36_for_plot.npy"), # for plotting only
         InputClass = gwta.FromTemplateBernoulliGenerator,
         inputSequence = np.repeat(np.arange(T), num_parallel).reshape(T, num_parallel) % len(template),
         inputKwargs = {"template" : template, "duration" : None},
         pspKwargs = {'kernel' : [1.]*tau, 'postprocess' : lambda y : np.clip(y, 0, 1)},    # rectangular, renewing PSPs
         ),
    "net" : dict(
        topo_pbc = True,
        dims = dims,
        K = K,
        N = N,
        tau = tau,
        kwargs_poszk = {"dist" : (3,2), "offset" : (1.5,0.5)},
        kwargs_posyi = {"dist" : (1,1), "offset" : (0,0)},
        kwargs_connV = {"connradius" : connradius_V, "norm" : "max-norm", "p_plast" : 1.0, "Vinit" : 0.0},
        kwargs_connW = {"connradius" : 0., "num_net" : num_subnet, "w" : 0.0, 
                        "norm" : "max-norm", "b_init" : -2.0, "W_inh" : -100.},
        VsynapseClass = gwta.BernoulliSynapses,
        VKwargs = {'syninit' : lambda k,i: 0., 'pi0i' : np.ones(N) * 0.10,
                   'semlike' : False, 'calcAk' : False, 'only_exc' : True},
        setV0_inpmean = False,
        set_piki_from_array = False,
        load_b_from_file = load_fname("b"),  
        load_V_from_file = load_fname("V"),  
        load_W_from_file = load_fname("W"),  
        samplerClass =  gwta.SamplerAbsref,
        samplerKwargs =  {'tau': tau, 'shuffleIdx': True },
        m_hom = 0.065, # = 0.72*... 0.8 * 0.5 * 4. / K, # 0.1  / (2*connradius_V+1)**2,
        ),
    "train" : dict(
        eta_V = 0.0,
        eta_b = 0.0,
        eta_W = 0.,
        eta_scale = lambda t: 0.0 * 1./tau, # * ( (0.01 - 1.)/float(T) * t + 1. ), # 1.0 * 1./tau, # 
        ),
}




