import numpy as np
import pylab as pl

import sys
sys.path.append('../')

import my_matplotlib_defaults
from figure_format_plos import fig_width, cm2inch
#from gwta.plotting import plot_styletrain
from gwta_shared import color as gc
from gwta_shared import plot_colored_styletrain
from sys import stdout

# # # # # # # # # # # # 
# # # P A R A M S # # # 
# # # # # # # # # # # # 

ZOOM = 1.5
SUBPLOTS = ("sketch", "template", "likelihood", "Vcomp", "projection", "weight", "spike")
SAVE = True

DSL = "Fig3_train" 

## Original paper parameters
snapshot_param = dict(T=100., dt=0.001,
                      times=(0,1000,3000,10000),
                      DSL = ("Fig3_eval1",
                             "Fig3_eval2",
                             "Fig3_eval3",
                             "Fig3_eval4")
                      )
## Parameters for quick testing
#snapshot_param = dict(T=100., dt=0.001,
#                      times=(0,50,100,200),
#                      DSL = ("Fig3_eval1",
#                             "Fig3_eval2",
#                             "Fig3_eval3",
#                             "Fig3_eval4")
#                      )


files = dict(ZS=tuple(), #("spike",), 
             YS=tuple(), #("spike",),
             Z=tuple(), #("projection",),
             Y=("inp_instance",), #  "projection"),
             V_mask=("weight", "projection", "spike", "Vcomp"),
             V0 = ("Vcomp", "Vcomp"),
             centers=("projection",),
             dims=("inp_instance",),
             Vtrace=("weight", "Vcomp"),
             ll=("likelihood",),
             ll_Ts=("likelihood",),
             #=("",),
       )


inp_instance_param = dict(times=(slice(850,1050),slice(1900,2100)))

inp_template_param = dict(fname="data/rate-diri03_7x3x36.npy",
                          posidx=3,
                          vmin=0.1,
                          vmax=0.6)

Vcomp_param = dict(t=snapshot_param['times'][-1],
                   droll = [6,0,0,0,0,0,-12],
                   V_theo = lambda piki,V0i: np.log(piki/(1.-piki)) - V0i,
                   dims = (7,3,36), 
                   X = np.load("./data/rate-diri03_7x3x36.npy").reshape(7*3, 36),
                   xmin = 0.1, xmax = 0.6,
                  )


projection_param = dict(times = snapshot_param['times'],
                        pca_tmax = 100000,
                        pca_dt = 10,
                        zidx = [9,10,11],
                        color=(gc.select['green'], gc.select['red'], gc.select['blue']),
                        ny_init = 10000,
                      )

weight_param = dict(rf_dims = (6,6),
                    idx_of_t = lambda t: int(t),
                   )


spike_param = dict(tslice = slice(26500,29000), # slice(26500,29000),
                   dt = 0.001,
                  )


likelihood_param = dict(t_max=10000.)


# # # # # # # # # # #
# # # S E T U P # # # 
# # # # # # # # # # # 

pl.interactive(False)


# Auto params
inp_instance_param['num'] = len(inp_instance_param['times'])
projection_param['num'] = len(projection_param['times'])


print " > DSL =", DSL
print " > Subplots:", SUBPLOTS

# Zooming
pl.rc("figure", dpi=ZOOM*pl.rcParams['figure.dpi'])

# Figure setup
fig = pl.figure(figsize=(fig_width['two_col'], cm2inch(13.)))
axes = dict()

lpad, rpad = 0.09, 0.03
wfig = 1. - lpad - rpad
ratio = fig.get_figwidth() / fig.get_figheight()

# add axes

# Sketch
dpi = rcParams['savefig.dpi']
w,h = 1100. / fig.get_figwidth()/dpi, 456. / fig.get_figheight() / dpi
rect = (lpad-0.06, 0.77, w, h)
axes["sketch"] = fig.add_axes(rect, aspect='equal', xticks=[], yticks=[])


# Input templates
l,w,h = lpad-0.06 + w + 0.06, 0.054, 0.054 * ratio
b = 0.95 - h
for i in xrange(3):
    rect = (l + (i % 2) * (w+ 0.02), b - (i>=2) * 1.35*h, w, h)
    axes["template_%d" % i] = fig.add_axes(rect, aspect='auto', xticks=[], yticks=[])
rect = (l + w+ 0.02, b - 1.35*h, 0.009, h)
axes["template_cb"] = fig.add_axes(rect, aspect='auto', xticks=[], yticks=[])


# Likelihood
rect = (0.605, 0.80, 0.755-0.605, 0.14)
axes["likelihood"] = fig.add_axes(rect, aspect='auto')

# Weight comparison
rect = (1-rpad-0.14, 0.80, 0.14, 0.14)
axes["Vcomp"] = fig.add_axes(rect, aspect='auto')


# Time arrow
rect = (lpad, 0.65, wfig, 0.06)
axes["timearrow"] = fig.add_axes(rect, xticks=[], yticks=[])

# Projection
n = projection_param['num']
d = 0.05
w = wfig / n - d * (n-1) / n
dw = 0.01
nz = len(projection_param['zidx'])
ww = w/3 - dw * (nz-1) / nz
for i in xrange(n):
    rect = (lpad+i*(w+d), 0.52-0.05, w, 0.13)
    axes["pca_%d" % i] = fig.add_axes(rect, aspect='auto', xticks=[], yticks=[])
    for j in xrange(nz):
        rect = (lpad+i*(w+d)+j*(ww+dw), 0.42-0.05, ww, ww*ratio)
        axes["weight_%d%d" % (i,j)] = fig.add_axes(rect, aspect='equal', xticks=[], yticks=[])
    # z spikes
    rect = (lpad+i*(w+d), 0.25-0.02, w, 0.11)
    axes["ZS_%d" % i] = fig.add_axes(rect, aspect='auto', xticks=[], yticks=[])
    # yspikes
    rect = (lpad+i*(w+d), 0.05, w, 0.15)
    axes["YS_%d" % i] = fig.add_axes(rect, aspect='auto', xticks=[], yticks=[])


for ax in fig.axes:
    ax.set_zorder(1)

# Overlay
rect = (0.0002, 0.0002, 0.9996, 0.9996)
axes["all"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=False, zorder=0)
letters = dict(A=(0.02,0.96), B=(0.32,0.96), C=(0.52,0.96), D=(0.78,0.96),
               E=(0.02,0.60), F=(0.02,0.44), G=(0.02,0.34))
for l,(x,y) in letters.iteritems():
    axes["all"].text(x, y, l, fontsize=8, fontweight='bold', ha='center', va='baseline')


# # # # # # # # # # # # # # # 
# # # L O A D   D A T A # # # 
# # # # # # # # # # # # # # # 


print " > Load data."
fname = lambda s: "out/"+DSL+"/"+s+".npy"
archive = np.load("out/"+DSL+"/archive.npz")
for key in files:
    if np.intersect1d(files[key], SUBPLOTS).size > 0:
        print "   > Attempt to load", key, "from", fname(key),
        try:
            if key in archive.keys():
                globals()[key] = archive[key]
            else:
                globals()[key] = np.load(fname(key))
            print "[ok]."
        except Exception, e:
            print "[FAILED]."
            print "     >", repr(e)
    else:
        print "   >", key, "data not needed for selected subplots."


# snapshot data
if np.intersect1d(("projection", "spike",), SUBPLOTS).size > 0:
    # load Y, YS, Z, ZS
    data_keys = ['Y', 'YS_compressed', 'Z', 'ZS_compressed']
    fname = lambda dsl, s: "out/%s/%s.npy" % (dsl, s)
    archivename = lambda dsl: "out/%s/archive.npz" % dsl
    snapshot = dict()
    p  = snapshot_param
    for t,dsl in zip(p['times'], p['DSL']):
        print "   > Load snapshot data for t=%d from DSL '%s'" % (t, dsl)
        snapshot[t] = dict()
        archive_snap = np.load(archivename(dsl))
        for key in data_keys:
            # load the data
            if key in archive_snap.keys():
                x = archive_snap[key]
            else:
                x = np.load(fname(dsl,key))
            # store it appropriately
            if key in ("Y", "Z"):
                x = x[:,0].astype(float)
            elif key in ("YS_compressed", "ZS_compressed"):
                T = int(round(p['T'] / p['dt']))
                K = max(x[2]) + 1
                x2 = np.zeros((T,K), dtype=bool)
                tn, k = x[0][x[1]==0], x[2][x[1]==0]
                x2[tn,k] = True
                x = x2
                key = key[:2]
            snapshot[t][key] = x


# post process
print "   > Post processing loaded data."
if "ZS" in globals():
    K = ZS.shape[-1]
    T = ZS.shape[0]
    ZS.resize((T,K)) # one network only
    #ZS = ZS[:,0]    # one network only
if "Z" in globals():
    K = Z.shape[-1]
    T = Z.shape[0]
    Z.resize((T,K))  # one network only
    #Z = Z[:,0]    # one network only
if "YS" in globals():
    N = YS.shape[-1]
    T = YS.shape[0]
    YS.resize((T,N)) # one network only
    #YS = YS[:,0]    # one network only
if "Y" in globals():
    N = Y.shape[-1]
    T = Y.shape[0]
    Y.resize((T,N))  # one network only
    Y = Y.astype(float)
    #Y = Y[:,0].astype(float)    # one network only
if "Vtrace" in globals():
    K,N = Vtrace.shape[1:]


# # # # # # # # # # # # #
# # #   T O O L S   # # #
# # # # # # # # # # # # #


class Projector(pl.matplotlib.mlab.PCA):
    """
    PCA class that builds upon input the neurons are tuned to at the end of training.
    """
    def __init__(self, Y, Z, zidx, yidx, ny_init):
        print " > Init projector."
        T = len(Z)
        #t = Z[-T/10:,zidx].nonzero()[0]     # response times
        t = Z[:,zidx].nonzero()[0]     # response times
        t = t.copy()
        print "   > Projector: Found %d non-zero responses." % len(t)
        np.random.shuffle(t)
        #y = Y[-T/10:,yidx][t[:ny_init]]     # random subset
        y = Y[:,yidx][t[:ny_init]]     # random subset
        pl.matplotlib.mlab.PCA.__init__(self, y)
        #self.Z = Z
        #self.Y = Y
        self.zidx = zidx
        self.yidx = yidx

    def project_by_response(self, Z, Y, tslice, k):
        """
        Projection of input in time slice. Restrict to input when response is:
          k = int : if zk = 1
          k = None: if all zk \in {zidx} are 0
        """
        if isinstance(k, int):
            assert k in self.zidx
            #t = self.Z[tslice,k] == 1
            t = Z[tslice,k] == 1
        elif k is None:
            #t = (self.Z[tslice,self.zidx] == 0).all(1)
            t = (Z[tslice,self.zidx] == 0).all(1)
        else:
            raise Exception
        y = Y[tslice,self.yidx][t]
        xy = self.project(y).T[:2]
        return xy


if "projection" in SUBPLOTS:
    p = projection_param
    zidx = p['zidx']
    yidx = V_mask[zidx[0]].nonzero()[0]
    z = snapshot[p['times'][-1]]['Z']
    y = snapshot[p['times'][-1]]['Y']
    pca = Projector(y, z, zidx, yidx, p['ny_init'])


def draw_bg_box(i):
    trans = axes["timearrow"].transData.transform
    inv = axes['all'].transData.inverted().transform
    x0,y0 = inv(trans([projection_param['times'][i],-0.15]))
    trans = fig.transFigure.transform
    w,d = 0.1825,0.05
    (x1,y1) = (x2,y4) = inv(trans([lpad+i*(w+d) - 0.033 - (i==0)*0.01, 0.61 + i * 0.007]))
    (x3,y2) = (x4,y3) = inv(trans([lpad+i*(w+d) + w + 0.008 + (i==3)*0.003, 0.03]))
    xy = np.array([(x0,y0),(x1,y1),(x2,y2),(x3,y3),(x4,y4)])
    ax = axes['all']
    p = pl.Polygon(xy, fc='0.9', ec='0.75', lw=0.5, zorder=0)
    ax.add_patch(p)
    ax.set_xlim(0,1)
    ax.set_ylim(0,1)


# # # # # # # # # # # #
# # #   P L O T   # # #
# # # # # # # # # # # #


if "sketch" in SUBPLOTS:
    print " > Render sketch."
    for ax,f in ([axes["sketch"], "fig/fig3a.png"],):
        im = pl.imread(f)
        print "  > Image %s has dims" % f, im.shape[:2]
        ax.imshow(im, interpolation='lanczos')
        ax.set_frame_on(False)




if "inp_instance" in SUBPLOTS:
    print " > Render inp_instance."
    for i in xrange(inp_instance_param['num']):
        ax = axes["ymean_%d" % i]
        s = inp_instance_param['times'][i]
        y = Y[s].mean(0)
        mappable = ax.imshow(y.reshape(dims).T, vmin=0, vmax=1, cmap=gc.input_cmap, origin='lower')
        ax.set_aspect("equal")
    ymean_cb = pl.colorbar(mappable=mappable, cax=axes["ymean_cb"])
    ymean_cb.set_label("Avg. input")
    ymean_cb.set_ticks(np.linspace(0,1,5))


if "template" in SUBPLOTS:
    print " > Render template."
    p = inp_template_param
    template = np.load(p['fname'])[p['posidx']]
    vmin, vmax = p['vmin'], p['vmax']
    template = vmin + (vmax - vmin) * template
    dims = weight_param['rf_dims']
    for i in xrange(3):
        ax = axes["template_%d" % i]
        mappable = ax.imshow(template[i].reshape(dims).T, vmin=vmin, vmax=vmax, cmap=gc.input_cmap, origin='lower')
        ax.set_aspect('equal')
        pl.setp(ax.spines.values(), color='0.3', lw=1.)
    ax = axes["template_cb"]
    template_cb = pl.colorbar(mappable, cax=ax, orientation="vertical")
    template_cb.set_label("Avg. input")
    template_cb.set_ticks(np.linspace(vmin,vmax,2))


if "likelihood" in SUBPLOTS:
    print " > Render panel likelihood."
    ax = axes['likelihood']
    p = likelihood_param
    # plot
    ax.plot(ll_Ts[ll_Ts<=p['t_max']], ll[ll_Ts<=p['t_max']], 'r', lw=1.0)
    for t in projection_param['times']:
        y = ll[t == ll_Ts][0]
        ax.plot([t,t], [ll.min(),y], ':', c='0.3')
    #ax.plot(p['times'], ll, 'ro', mew=0, mec='r', ms=2)
    ax.set_xticks([0,5000,10000])
    ax.set_xlim(0, 1.06*p['t_max'])
    ax.set_xlabel("Learning time [s]", labelpad=3)
    ax.set_ylabel("Log-likelihood")
    ymin, ymax = ax.get_ylim()
    ax.set_yticks(np.round(np.linspace(ymin,ymax,4)))
    # make axes nice
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.yaxis.set_tick_params(size=1)
    ax.xaxis.set_tick_params(size=1)
    ax.get_xaxis().tick_bottom()
    ax.get_yaxis().tick_left()


if "Vcomp" in SUBPLOTS:
    print " > Render panel V comparison"
    ax = axes['Vcomp']
    p = Vcomp_param
    V = Vtrace[p['t']]
    X = (p['xmax'] - p['xmin']) * p['X'] + p['xmin']
    droll = p['droll']
    V_theo = p['V_theo']
    nSyn = np.prod(p['dims'])
    # Do the weight assignment
    xy = M = np.zeros(p['dims'] + (2,))
    for l in xrange(p['dims'][0]):       # iterate over locations
        ks = range(p['dims'][1] * l, p['dims'][1] * (l+1))
        v_theo = V_theo(X[ks], V0[V_mask[ks[0]]])
        for k in ks:
            v = V[k][V_mask[k]]
            v = np.roll(v, droll[l])
            j = np.dot(v_theo,v).argmax()    # best matching input
            xy[l,k % p['dims'][1],:,0] = v
            xy[l,k % p['dims'][1],:,1] = v_theo[j]
    # plotting
    x = xy.reshape(nSyn,2)[:,0]
    y = xy.reshape(nSyn,2)[:,1]
    ax.plot(x, y, '.', ms=0.5, c=gc.weight_cmap(0.3))
    ax.plot([0,2.8],[0,2.8], '0.3', lw=0.5, alpha=0.6)
    ax.set_xticks([0,1,2,3])
    ax.set_yticks([0,1,2,3])
    ax.set_xlim(0, np.ceil(xy.max()))
    ax.set_ylim(0, np.ceil(xy.max()))
    ax.set_aspect('equal')
    ax.set_xlabel("Weight V$_\mathsf{ki}$", labelpad=3)
    ax.set_ylabel("Optimal weight")
    # make axes nice
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.yaxis.set_tick_params(size=1)
    ax.xaxis.set_tick_params(size=1)
    ax.get_xaxis().tick_bottom()
    ax.get_yaxis().tick_left()


if "projection" in SUBPLOTS:
    print " > Render panel projection."
    # Timearrow
    ax = axes["timearrow"]
    times = projection_param['times']
    p = pl.matplotlib.patches.FancyArrow(x=0,y=0,dx=1.1*times[-1],dy=0,
          length_includes_head=True, head_width=0.3, head_length=0.01*times[-1], width=0.0,
          fc='k', ec='k', lw=0.5)
    ax.add_patch(p)
    for t in times:
        ax.plot([t,t],[-0.15,0.15],'k', lw=0.5)
        ax.text(t, 0.5, str(t), ha='center', va='baseline')
    ax.text(0.55*times[-1], 0.5, "Learning time [s]", ha='center', va='baseline')
    ax.set_xlim(0,1.11*times[-1])
    ax.set_ylim(-2,1)
    ax.set_frame_on(False)
    # draw gray boxes
    for i in xrange(len(projection_param['times'])):
        draw_bg_box(i)
    # projections
    p = projection_param
    for i,t in enumerate(p['times']):
        ax = axes["pca_%d" % i]
        ax.set_xticks([])
        ax.set_yticks([])
        pca_tmax, pca_dt = p['pca_tmax'], p['pca_dt']
        #tslice = slice(t,t+pca_tmax,pca_dt)
        tslice = slice(0,pca_tmax,pca_dt)
        # No response
        z = snapshot[t]['Z']
        y = snapshot[t]['Y']
        xy = pca.project_by_response(z, y, tslice, k=None)
        print "   > Projecting %d points w/o response" % xy.shape[1]
        ax.plot(xy[0], xy[1], '.', c="0.5", ms=0.5)
        if i == 0:
            ax.set_xlabel("PC1")
            ax.set_ylabel("PC2", labelpad=8)
            ax.xaxis.set_label_position('top') 
        # z response
        for k,c in zip(p['zidx'],p['color']):
            xy = pca.project_by_response(z, y, tslice, k=k)
            print "   > Projecting %d points with k=%d responding" % (xy.shape[1], k)
            ax.plot(xy[0], xy[1], '.', c=c, ms=0.5)


if "weight" in SUBPLOTS:
    print " > Render panel weight."
    pp = projection_param
    pw = weight_param
    vmax = Vtrace.max()
    for i,t in enumerate(pp['times']):
        for j,(k,c) in enumerate(zip(pp['zidx'],pp['color'])):
            ax = axes["weight_%d%d" % (i,j)]
            tn = pw['idx_of_t'](t)
            w = Vtrace[tn,k][V_mask[k]].reshape(pw['rf_dims'])      # WARNING: Error prone at edges!
            ax.imshow(w.T, vmin=0, vmax=vmax, cmap=gc.weight_cmap, origin='lower')
            pl.setp(ax.spines.values(), color=c, lw=1.)
            a,b = ax.get_xlim()
            ax.set_xlim(a-0.2,b+0.2)
            ax.set_xticks([])
            a,b = ax.get_ylim()
            ax.set_ylim(a-0.2,b+0.2)
            ax.set_yticks([])
            if i == j == 0:
                ax.set_ylabel("V$_\mathsf{ki}$", labelpad=8)




if "spike" in SUBPLOTS:
    #raise Exception
    print " > Render panel spike."
    pp = projection_param
    #assert pp['times'][0] == 0, " > ERROR: First spike pattern should start at 0s!"
    ps = spike_param
    for i,t in enumerate(pp['times']):
        # z spikes
        ax = axes['ZS_%d' % i]
        #zs = ZS[t:][ps['tslice']]
        zs = snapshot[t]['ZS'][ps['tslice']]
        t_max = (ps['tslice'].stop - ps['tslice'].start) * ps['dt']
        ctemplate=zs[:,pp['zidx']]
        cidx = np.array(pp['zidx']).reshape(-1,1)
        color=pp['color']
        plot_colored_styletrain(zs, t_min=0., t_max=t_max, dt=ps['dt'],
                                ctemplate=ctemplate, cidx=cidx, color=color,
                                ms=1.5, marker='|', default_color='0.2',
                                bgcolor=gc.network_spikebg, show_scale=False, ax=ax)
        if i == 0:
            ax.set_ylabel("Network z$_\mathsf{k}$", labelpad=0)
        # y spikes
        ax = axes['YS_%d' % i]
        #ys = YS[t:][ps['tslice']]
        ys = snapshot[t]['YS'][ps['tslice']]
        t_max = (ps['tslice'].stop - ps['tslice'].start) * ps['dt']
        yidx = V_mask[pp['zidx'][0]].nonzero()[0]
        ctemplate=ys[:,yidx]
        cidx = np.array(yidx).reshape(-1,1)
        color=["k"]*len(yidx)
        plot_colored_styletrain(ys, t_min=0., t_max=t_max, dt=ps['dt'], \
                                ctemplate=ctemplate, cidx=cidx, color=color, \
                                ms=0.3, marker='.', default_color="k", mew=0.3, \
                                bgcolor=gc.input_spikebg, show_scale=(t==pp['times'][0]),
                                t_scale=1.0, ax=ax)
        if i == 0:
            ax.set_ylabel("Input y$_\mathsf{i}$", labelpad=-5)


# # # # # # # # # # # # # #
# # # F I N A L I Z E # # #
# # # # # # # # # # # # # #
pl.interactive(True)
pl.draw()


if SAVE is True:
    print " > Save figure to 'fig/Figure.{png,pdf}'."
    fig.savefig("fig/Figure.png")
    fig.savefig("fig/Figure.pdf")

print " > Show figure."
pl.show()
print " > Done."

