import numpy as np
import pylab as pl
import time
from sys import stdout
import sys
sys.path.append('../')

from liveplot import LivePlot
import my_matplotlib_defaults
import gwta_shared as gwta

# # # INIT (extract and save params) # # #
paramfilename = gwta.get_param_file()
print " > Loading parameters from file " + paramfilename + ".py"
exec("from " + paramfilename + " import param")     # load parameter dictionary
gwta.extract_params(param, globals(), keep=('samplerKwargs', 'kwargs_poszk', 'kwargs_posyi', 'kwargs_connV', 'kwargs_connW', 'inputKwargs', 'pspKwargs', 'VKwargs'))

print " > DSL is", DSL
print " > Simulation will" + (spiking and ' ' or ' NOT ') + "be spiking."

# set random seed
np.random.seed(seed)

# create logger for sparsely saved data
kwargs = dict(Wtrace=("W", (K,K), 'float'), btrace=("b", (K,), 'float'), Vtrace=("V", (K,N), 'float'))
logger = gwta.Logger(DSL, T, SAVEEVERY, TEST, globals(), **kwargs)
logger.save_param(paramfilename)


# # # GENERATE NETWORK # # #

# Build network structure
net = gwta.NetworkBuilder(K, N, dims, pbc=topo_pbc)
rdimsz = net.set_pos_raster("zk", **kwargs_poszk)
rdimsy = net.set_pos_raster("yi", **kwargs_posyi)
net.set_V(**kwargs_connV)
net.set_W(**kwargs_connW)
subnet = net.add_subnet(**kwargs_connW)
net.check()

# neuron and synapse arrays
V = VsynapseClass(K=K, N=N, mask=net.V_mask, **VKwargs)     # input synapse object
W = gwta.shmarray.zeros((K,K), dtype='d')                   # shared sampler matrix
b = gwta.shmarray.zeros(K, dtype='d')                       # shared sampler vector
V0 = V.get_V0()
ext = np.zeros((num_parallel,K))                            # vector for summed external input
V[:] = net.V
V_mask = net.V_mask
W[:] = net.W
W_mask = net.W_mask
b[:] = net.b

# load synapses and biases from file
if load_V_from_file:
    print " > Loading V from file '%s'." % load_V_from_file
    V.W[:] = np.load(load_V_from_file)

if load_b_from_file:
    print " > Loading b from file '%s'." % load_b_from_file
    b[:] = np.load(load_b_from_file)

if load_W_from_file:
    print " > Loading W from file '%s'." % load_W_from_file
    i = load_W_from_file.find("/W") + 2
    W_mask_fname = load_W_from_file[:i] + "_mask" + load_W_from_file[i:]
    #W_mask_fname = load_W_from_file[:-4] + "_mask.npy"
    W_mask[:] = np.load(W_mask_fname)
    W[W_mask] = np.load(load_W_from_file)[W_mask]

# init parallel processes
sampler = samplerClass(W, b, **samplerKwargs)
co = gwta.Coordinator(sampler, num_parallel, spiking)
num_parallel = co.num_parallel


# # # MEMORY CHECK # # #
# memory check for full array (Z, Y, shownTemplate, maybe: spikes)
tmpa = [((num_parallel,K),"int8"),((num_parallel,N),"int"),((num_parallel,),"int")]
if spiking: tmpa += [((num_parallel,K),"int8"), ((num_parallel,N),"int")]
gwta.memorycheck(T=T, arrays=tmpa, limit=0.9)


# # # GENERATE INPUT # # #
if NOINPUT:
    print " > WARNING: Input is switched off! Only zeros are presented!"
    try:
        template = inputKwargs['template']
    except:
        print " > Warning: No input templates defined."
    Y = np.zeros((T,num_parallel,N), dtype=np.float16)
    YT = Y
    if spiking:
        YS = Y
else:
    print " > Generating input of shape (%d,%d,%d)" % (T, num_parallel, N)
    try:
        template = inputKwargs['template']
    except:
        print " > Warning: No input templates defined."
    inpgen = InputClass(num_parallel=num_parallel, spiking=spiking, tau=tau, **inputKwargs)
    if 'inputSequence' not in locals():
        inputSequence = None
    if spiking:
        #print "WARNING: HACK!"
        #seq = np.array(([7]*500 + [112]*500 + [71]*500)*40).reshape(60000,1)
        #YT, YS = inpgen.generate(T, sequence=seq)
        YT, YS = inpgen.generate(T, sequence=inputSequence)
        print " > Applying PSP kernel."
        Y = gwta.applyFastBinaryKernel(YS, tau, outdtype=np.float16)
        # Y = gwta.applyPSPkernel(S=YS, dtype=np.float16, **pspKwargs)
    else:
        YT, Y = inpgen.generate(T, sequence=inputSequence)

# Adapt V0
if setV0_inpmean:
    print " > Setting V0 to match input marginals."
    V.set_V0(V._logit(Y[:100000].mean(1).astype(float).mean(0)))
    V0 = V.get_V0()

if set_piki_from_array is not False:
    print " > Setting V from piki-array."
    piki = set_piki_from_array
    assert piki.shape == (K,N)
    V.set_centers(piki)
    if VKwargs['only_exc'] is True:
        np.clip(V.W, 0., 1000., out=V.W)



# # # RESULT ARRAYS # # #
Z = np.zeros((T,num_parallel,K), dtype=np.int8)
if spiking:
    ZS = np.zeros((T,num_parallel,K), dtype=np.int8)


# # # START SIM # # #
# start samplers
co.start_samplers()

# last preparations
if PLOTEVERY:
    plotter = LivePlot(sim_globals=globals())

ETA = lambda t: t and ( (float(T) / t - 1.) * (time.time() - time_start) )
reportmodulo = int(round(1./dt))
pe = gwta.ParameterEvolution(Nfilter=10, V=V.W, b=b, W=W)
print "\n > Start simulation of %.1fs (%d samples per sampler)" % (T_si, T)
time_start = time.time()


# # # MAIN LOOP # # #
for t in xrange(T):
    if t % reportmodulo == 0:
        print " > Time t=%ds \t ETA %.1fs" % (int(round(t*dt)), ETA(t))
        print "  > Mean(V) = %.2e" % (V[V_mask].mean())
        print "  > Mean(W) = %.2e" % (W[W_mask].mean())
        pe.get_stats(V=V.W, b=b, W=W)
    if PLOTEVERY and t % PLOTEVERY == 0:
        print " > plotting...",; stdout.flush()
        plotter.update()
        print "Done."; stdout.flush()
    # distribute input
    for i in xrange(num_parallel):
        ext[i] = V.transmit(Y[t,i])
        #if max(ext[i]) + kwargs_connW["W_inh"] > 0.:
            #print " > WARNING: Input exceeded inhibition at time %d." % t
    co.distribute_ext(ext)
    # unleash samplers
    co.sampling_initiate()
    # learning on previous states
    if Tplast[0] < t <= Tplast[1]:
        if eta_V > 0.:
            V.plasticity(eta_scale(t)*eta_V, Z[t-1], Y[t-1])
            #for i in xrange(num_parallel): V.plasticity(eta_scale(t)*eta_V, Z[t-1,i], Y[t-1,i])
            #V.plasticity(eta_scale(t)*eta_V*num_parallel, Z[t-1].mean(0), Y[t-1].mean(0))
            ##V0 = np.log(V.param['lambda0i'])
            #V[:] += eta_scale(t) * eta_V * V_mask * ( np.exp(-V.W - V0) * np.dot(Z[t-1].T,Y[t-1]) - Z[t-1].sum(0)[:,None] )
        if eta_b > 0.:
            b += eta_scale(t) * eta_b * num_parallel * (m_hom - Z[t-1].mean(0))
        if eta_W > 0.:
            W += eta_scale(t) * eta_W * W_mask * rec_plast_rule(W, Z[t-1], m_hom, alpha=1., exponent=2)
            if only_exc_W:
                W[:] = (1-W_mask) * W + W_mask * np.clip(W, 0., 10000.)
    # collect samples
    if spiking:
        Z[t], ZS[t] = co.sampling_collect()
    else:
        Z[t] = co.sampling_collect()
    # track results
    logger.track(t)


# # # FINALIZE AND SAVE RESULTS # # #
print " > Simulation took %.1fs" % (time.time() - time_start)
if PLOTEVERY:
    plotter.update()
co.finalize()

centers = V.get_centers()
Ak = V.get_Ak()

kwargs = dict(individual = ["subnet", "dims", "rdimsz", "rdimsy", "V0",
                            "V", "V_mask", "W", "W_mask", "b", "centers", "Ak"],
              archive = [])

if globals().has_key("template"):
    kwargs['individual'] += ["template"]

# Check size limit:
SIZELIMIT_GB = 2.
limited_list = ["Y", "YT", "Z", "Wtrace", "btrace", "Vtrace"]
for key in limited_list:
    if globals().has_key(key):
        x = globals()[key]
    else:
        x = logger.__getattribute__(key)
    if x.nbytes / 2.**30 < SIZELIMIT_GB:
        kwargs['archive'] += [key]
    else:
        print " > Warning: excluding '%s' from archive (larger than %.1f GB)." % (key, SIZELIMIT_GB)


if spiking:
    print " > Compress input spikes."
    t, p, i = YS.nonzero()
    n = YS[(t,p,i)]
    YS_compressed = np.vstack((t,p,i,n)).astype(np.int32)
    kwargs['archive'].append("YS_compressed")
    print " > Compress network spikes."
    t, p, i = ZS.nonzero()
    n = ZS[(t,p,i)]
    ZS_compressed = np.vstack((t,p,i,n)).astype(np.int32)
    kwargs['archive'].append("ZS_compressed")

logger.save(**kwargs)

#p = np.mean([gwta.joint_from_data(Z[:,i])[1] for i in xrange(num_parallel)], 0)
#q = gwta.joint_of_BM(b+ext[0], W)[1]




