import sys
sys.path.append('../')
import numpy as np

"""
Calc log-likelihood in a gwta setup. The data distribution is represented by M samples Y.
p(Z) is calculated analytically over array of allowed states Z. We can assume a value for beta_k, 
i.e. we can calculate the likelihood of the data under the model the network makes inference in.
Then the prior parameters are estimated as \hat b_k = b_k + A_k + assume_beta.
If assume_beta is None: \hat b_k must be present explicitely as b_priortrace.

We find L = 1/M \sum_y log p(y)
        log p(y) = log(\prod_i h_i(y_i)) + V0*y - A0*1 - log(Z_partition) +
                   + log \sum_z exp( z' * ( 1/2 W*z + V*y + \hat b - A ) )

"""

# # # # # # # # # #
# # P A R A M S # #
# # # # # # # # # #

# w/o rec conns
DSL = "Fig4_train_local"

DSL2 = None

fname = lambda s: "out/%s/%s.npy" % (DSL, s)
fname2 = lambda s: "out/%s/%s.npy" % (DSL2, s)

Ts = np.array([0, 300, 500, 800, 1000, 1300, 1500, 1800, 2000, 2300, 2500, 2800, 3000, 3300, 3500, 3800, 4000, 5000, 6000, 7500, 10000, 12500, 15000, 17500, 20000, 22500, 24999]) 
Z = np.load("data/allowed_z_states.npy") # list of allowed z-states
Y = np.load("out/Y_samples.npy").astype(np.float64)  # data samples

recalculate_hatb = True
m_hom = 0.95 * 1./3.


assume_beta = None # None or value; if None: b_priortrace must be present.

# For Bernoulli model
h = lambda x: np.ones(x.shape)
A = lambda x: np.log(1 + np.exp(x))


# # # # # # # # # #
# # # L O A D # # #
# # # # # # # # # #

print " > Loading data set '%s'" % DSL

V0 = np.load(fname("V0"))
archive = np.load(fname("archive")[:-1] + "z")

Vtrace = archive["Vtrace"]
Wtrace = archive["Wtrace"]
btrace = archive["btrace"]

if DSL2 is not None:
    archive2 = np.load(fname2("archive")[:-1] + "z")
    Vtrace = np.append(Vtrace, archive2["Vtrace"], axis=0)
    Wtrace = np.append(Wtrace, archive2["Wtrace"], axis=0)
    btrace = np.append(btrace, archive2["btrace"], axis=0)



assert not ((assume_beta is not None) and (recalculate_hatb is True))
if (assume_beta is None) and (recalculate_hatb is False):
    print "  > Loading b_priortrace from file."
    b_priortrace = archive["b_priortrace"]
    if DSL2 is not None:
        b_priortrace = np.append(b_priortrace, archive2["b_priortrace"], axis=0)



# # # # # # # # # #
# # V A L U E S # #
# # # # # # # # # #

assert Ts.max() <= Vtrace.shape[0]
K, N = Vtrace.shape[1:]
M = Y.shape[0]
S = Z.shape[0]

global log_prod_hiY     # float (not devided by M)
global V0Y_minus_A0     # float (not devided by M)

global hatb                     # dims = (K,)
global half_ZWZ_plus_Zb         # dims = (S,)
global log_Z_partition          # float

global ZVY                      # dims = (S,M)

# # # # # # # # # # # # #
# # F U N C T I O N S # #
# # # # # # # # # # # # #

def adapt_hatb(b, W, Z, m_hom, numIter=10):
    from gwta_shared import joint_of_BM_indexed, calc_marginals_indexed
    b = b.copy()
    eta_b = 1.0
    for i in xrange(numIter):
        L,P = joint_of_BM_indexed(b,W,Z)
        p = calc_marginals_indexed(P,Z)
        b += eta_b * (m_hom - p)
    return b


def calc_hatb(b, A):
    global hatb
    hatb = b + A - assume_beta

def calc_fully_globals(Y, V0):
    global log_prod_hiY
    global V0Y_minus_A0
    log_prod_hiY = np.log(h(Y)).sum()
    V0Y_minus_A0 = np.sum(np.dot(Y, V0) - A(V0).sum())


def calc_per_time_globals(Z, W, b, V, V0):
    global hatb
    global half_ZWZ_plus_Zb
    global log_Z_partition
    Ak = np.sum(A(V+V0) - A(V0), axis=1)
    if hatb is None: calc_hatb(b, Ak)
    half_ZWZ_plus_Zb = np.array([ np.dot(wz,z) for (wz,z) in zip(np.dot(Z, 0.5 * W.T) + hatb - Ak, Z) ]) 
    dummy = np.array([ np.dot(wz,z) for (wz,z) in zip(np.dot(Z, 0.5 * W.T) + hatb, Z) ])
    log_Z_partition = np.log(np.exp(dummy).sum(0))


def calc_locals(Y, Z, V):
    global ZVY
    ZVY = np.dot(Z, np.dot(V, Y.T))


def calc_log_likelihood(Y, Z, W, b, V, V0):
    calc_per_time_globals(Z, W, b, V, V0)
    calc_locals(Y, Z, V)
    logsumzexpzvmem = np.log(np.exp(half_ZWZ_plus_Zb[:,None] + ZVY).sum(0))
    ll = (log_prod_hiY + V0Y_minus_A0) / M + (logsumzexpzvmem - log_Z_partition).mean()
    return ll



# # # # # # # # # #
# # # M A I N # # #
# # # # # # # # # #

if recalculate_hatb:
    from sys import stdout
    print " > Recalc b_priortrace."
    b_priortrace = np.zeros(btrace.shape)
    b = btrace[0]
    numIter = 400
    for T in Ts:
        print "\b.",; stdout.flush()
        W = Wtrace[T]
        b = adapt_hatb(b, W, Z, m_hom, numIter=numIter)
        b_priortrace[T] = b
        numIter = 100
    print ""

#raise Exception

print " > Calc LogLikelihood."
calc_fully_globals(Y, V0)

ll = []
for T in Ts:
    print " > T = %d" % T
    W, b, V = Wtrace[T], btrace[T], Vtrace[T]
    if assume_beta is None:
        hatb = b_priortrace[T]
    else:
        hatb = None
    ll.append(calc_log_likelihood(Y, Z, W, b, V, V0))

print " > Save results."
np.save(fname('ll'), ll)
np.save(fname('ll_Ts'), Ts)










