from liveplot.panel import *
import numpy as np

FIGSIZE = (14,8)
DPI = 87

LAYOUT = [[7,7,6,6,4,4],
          [7,7,6,6,5,5],
          [7,7,6,6,5,5],
          [2,2,8,8,3,1]]

#LAYOUT = [[1,1,0,0,0],
          #[1,1,2,2,2],
          #[1,1,2,2,2],
          #[3,3,5,6,6]]


status = dict(
    attach = ('DSL', 't', 'T'),
    text = lambda : "Simulation %s  (time t=%d of %d)" % (DSL, t, T-1)
    )

panels = {
    0 : dict(panelClass = EmptyPanel),
    1 : dict(panelClass = BarPanel,
             attach = ('t', 'Z', 'm_hom'),
             initKwargs = lambda : dict(xlabel="Neuron k", ylabel="<z_k>", target=m_hom, barcolor='0.7'),
             updateKwargs = lambda: dict(data=Z[max(0,t-20000):t+1].mean(0).mean(0), title="Mean activity (20s)")
        ),
    2 : dict(panelClass = HistogramPanel,
             attach = ('V', 'V_mask'),
             initKwargs = lambda : dict(xlabel='V_ki', ylabel='Num', trace=True, 
                                        log=False, normed=False, barcolor='cadetblue'),
             updateKwargs = lambda: dict(data=V[V_mask], bins=50, title="Input weight distribution")
        ),
    3 : dict(panelClass = ImageGridPanel,
             attach = ('b', 'rdimsz'),
             initKwargs = lambda : dict(symmetric=False, vrange=(None,None), origin='lower', 
                                        colorbar=True, cblabel='b_k'),
             updateKwargs = lambda: dict(data=b.reshape(rdimsz), title="Biases")
        ),
    4 : dict(panelClass = SpiketrainPanel,
             attach = ('ZS', 't'),
             initKwargs = lambda : dict(t_min=0.0, t_max=2.0, dt=0.001, tau=None, ms=3.0),
             updateKwargs = lambda: dict(data=ZS[max(0,t-2000):t+1,0], title="Output spikes")
        ),
    5 : dict(panelClass = SpiketrainPanel,
             attach = ('YS', 't'),
             initKwargs = lambda : dict(t_min=0.0, t_max=2.0, dt=0.001, tau=None, ms=0.5),
             updateKwargs = lambda: dict(data=YS[max(0,t-2000):t+1,0], title="Input spikes")
        ),
    6 : dict(panelClass = RecurrentConnectionsPanel,
             attach = ('W', 'W_mask', 'rdimsz'),
             initKwargs = lambda : dict(W_mask=W_mask, dims=rdimsz, maxconn=100, seed=0, legend=True),
             updateKwargs = lambda: dict(W=W, title="Recurrent weights")
        ),
    7 : dict(panelClass = ReceptiveFieldsPanel,
             attach = ('V', 'V_mask', 'rdimsz'),
             initKwargs = lambda : dict(post_dims=rdimsz, rf_dims=[int(np.sqrt(V_mask[0].sum()))]*2, V_mask=V_mask, 
                                        colorbar=True, cblabel='pi_ki'),
             updateKwargs = lambda: dict(data=V.get_centers(), title="Receptive field centers")
        ),
    8 : dict(panelClass = HistogramPanel,
             attach = ('W', 'W_mask'),
             initKwargs = lambda : dict(xlabel='W_kj', ylabel='Num', trace=True, 
                                        log=False, normed=False, barcolor='tomato'),
             updateKwargs = lambda: dict(data=W[W_mask], bins=30, title="Rercurrent weight distribution")
        ),
}