import numpy as np
import time
import sys
sys.path.append('../')
import my_matplotlib_defaults
import gwta_shared as gwta
import time
from billtool import generate_DSL

# # # # # # # # # # # # # # # # # # # # # # # #
# # # G E N E R A T I N G   P A R A M S # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

np.random.seed(0)       # seed for parameter generation
DSL = "Fig4_test_tst"           # Data Set Locator defines the directory name where
                            # results are stored within the "out/"-directory
#DSL = generate_DSL()        # Use this to generate a timestamp-DSL
#                            # If used, the DSL has to be changed in other scripts

T_si = 9*100. # 1000. #30.0                   # Total time in SI units
dt = 0.001                   # discretization time step in SI units
tau_si = 0.010               # tau in SI units

K = 7*3 # 20**2 - 20
N = 7*6*6 # 20**2
dims = (42,6) # (20,20)
connradius_V = 2.6 # 2.6
num_subnet = 3

THEOBASED = False
LOAD_DSL = "Fig4_train_local_tst"

rec_plast_rule = THEOBASED and gwta.wake_sleep_plasticity or gwta.covariance_plasticity_tan

template = np.load("data/nine_cues-7.npy")
template[template==0.5] = 0.5
template = template.reshape(template.shape[0], -1)
template = 0.1 + (0.6 - 0.1) * template

sequence = np.arange(9).repeat(100000, axis=0).reshape(-1,1)

# # # # # # # # # # # # # # # # # # # # # # # #
# # # # # P R E P R O C E S S I N G # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

tau = int(round(tau_si/dt))
T = int(round(T_si/dt))
load_fname = lambda s: LOAD_DSL and "./out/"+LOAD_DSL+"/"+s+".npy" or False



# # # # # # # # # # # # # # # # # # # # # # # #
# # # # # A C T U A L   P A R A M S # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

param = {
    "global" : dict(
        T_si = T_si,
        T = T,
        dt = dt,
        seed = None,
        DSL = DSL,
        num_parallel = 1,           # level of parallelization during learning (int or 'auto')
        spiking = True,             # whether to use spikes streams or independent samples
        Tplast = (0, T),            # plasticity will be switched on/off at these times 
        PRIOR = THEOBASED,                  # False or burnin; Also generate num_parallel independent Gibbs prior samplers?
        TEST = False,                # whether to save params and results
        PLOTEVERY = 20000,           # Plotter update, timesteps or None/0
        SAVEEVERY = 1000,            # save only every n-th step (for logger)
         ),
    "input" : dict(
         NOINPUT = False,            # present zeros only
         InputClass = gwta.FromTemplateBernoulliGenerator,
         inputKwargs = {"template" : template, "duration" : int(round(0.250/dt))},
         pspKwargs = {'kernel' : [1.]*tau, 'postprocess' : lambda y : np.clip(y, 0, 1)},    # rectangular, renewing PSPs
         inputSequence = sequence,
         ),
    "net" : dict(
        topo_pbc = False,
        dims = dims,
        K = K,
        N = N,
        tau = tau,
        kwargs_poszk = {"dist" : (6,2), "offset" : (2.5,0.5)},
        kwargs_posyi = {"dist" : (1,1), "offset" : (0,0)},
        kwargs_connV = {"connradius" : connradius_V, "norm" : "abs-x", "p_plast" : 1.0, "Vinit" : 0.0},
        kwargs_connW = {"connradius" : 100., "num_net" : num_subnet, "w" : 0., 
                        "norm" : "abs-x", "b_init" : -1., "W_inh" : -100.},
        VsynapseClass = gwta.BernoulliSynapses,
        VKwargs = {'syninit' : lambda k,i: 0., 'pi0i' : np.ones(N) * 0.1,
                   'semlike' : False, 'calcAk' : False, 'only_exc' : True},
        setV0_inpmean = False,
        load_b_prior_from_file = load_fname("b_prior"),  # "out/2014-06-03-11-56-38_b.npy",
        load_b_from_file = load_fname("b"),
        load_V_from_file = load_fname("V"),
        load_W_from_file = load_fname("W"),
        samplerClass =  gwta.SamplerAbsref,
        samplerKwargs =  {'tau': tau, 'shuffleIdx': True },
        m_hom = 0.95 * 1. / 3.,
        ),
    "train" : dict(
        eta_V = 0.0,
        eta_b = 0.0,
        eta_W = 0.0, # 0.001,
        rec_plast_rule = rec_plast_rule,  # gwta.correlation_plasticity,
        kwargs_rec_plast = {"A": 0.851, "beta" : 15.0},
        only_exc_W = True,
        eta_scale = lambda t: 0.2/tau, 
        ),
}




