# Using the magic encoding
# -*- coding: utf-8 -*-

import numpy as np
import pylab as pl
import sys
sys.path.append('../')
from figure_format_plos import fig_width, cm2inch
from gwta_shared.plotting import plot_styletrain, plot_colored_styletrain
from gwta_shared.builder import dist_pbc
from gwta_shared import color as gc

# # # # # # # # # # # # 
# # # P A R A M S # # # 
# # # # # # # # # # # # 

ZOOM = 1.5
SUBPLOTS = ("sketch","rec_weight","likelihood","inference_A","inference_B")
SAVE = True

DSL = "Fig4_test_tst"  # FROM INFERENCE



files = dict(ZS_compressed=("inference_A", "inference_B"),    # Files required by different subplots: DSL_KEY.npy=("A", "B")
             Z = ("inference_A", "inference_B"),
             W=("rec_weight", ),
             W_mask=("rec_weight", ),
             V = ("inference_A", "inference_B", "rec_weight", ),
             V_mask = ("inference_A", "inference_B", "rec_weight", ),
             rdimsz=("rec_weight", ),
             dims=("inference_A", "inference_B","rec_weight","sketch"),
            )


sketch_inp_param = dict(Y=np.load("out/Fig4_y_example/archive.npz")['Y'][:,0],
                        t_slice=slice(2*1000+500,2*1000+750),
                        example=[(0,2,1),(2,0,1),(2,0,2),(2,1,2)],
                        color=(gc.select['red'], gc.select['green'], gc.select['blue']))


rec_weight_param = dict(color=(gc.select['red'], gc.select['green'], gc.select['blue']),
                        idx=(6,), lw_min=0.2, lw_max=2.)


likelihood_param = dict(label=("Wake-sleep", "Simple rule"), dT=1., # SAVEEVERY in seconds
                        times=(## No simulation w/o rec exc. for simplicity
                               "out/Fig4_train_ws_tst/ll_Ts.npy",
                               "out/Fig4_train_local_tst/ll_Ts.npy"),
                        ll=(
                            "out/Fig4_train_ws_tst/ll.npy",
                            "out/Fig4_train_local_tst/ll.npy"),
                        color=("0.1", "r"), ls=("--", "-") )



inference_A_param = dict(ts0=102., ts1=103.5, dt=0.001, tm0=100., tm1=200.,
                         cue=(0,1), bgcolor = gc.network_spikebg,
                         color=(gc.select['red'], gc.select['green'], gc.select['blue']))

inference_B_param = dict(ts0=807.0, ts1=808.5, dt=0.001, tm0=800., tm1=900.,
                         cue=(2,2), bgcolor = gc.network_spikebg,
                         color=(gc.select['red'], gc.select['green'], gc.select['blue']))


# # # # # # # # # # #
# # # S E T U P # # # 
# # # # # # # # # # # 
pl.interactive(False)

print " > DSL =", DSL
print " > Subplots:", SUBPLOTS

# Zooming
pl.rc("figure", dpi=ZOOM*pl.rcParams['figure.dpi'])

# Figure setup
fig = pl.figure(figsize=(fig_width['two_col'], cm2inch(6.)))
axes = dict()

lpad, rpad = 0.03, 0.03
wfig = 1. - lpad - rpad
ratio = fig.get_figwidth() / fig.get_figheight()

# add axes
w1 = 0.32*wfig
w2 = 0.20*wfig
w3 = 0.32*wfig
d = 0.08 * wfig
h1 = w1 * ratio * 425. / 1232.
print " > Axes 'sketch_net' is %d x %d pixels." % (w1 * fig.get_figwidth() * rcParams['savefig.dpi'],
                                                   h1 * fig.get_figheight() * rcParams['savefig.dpi'])
rect = (lpad, 0.63, w1, h1)
axes["sketch_net"] = fig.add_axes(rect, xticks=[], yticks=[])

ne = len(sketch_inp_param['example']) + 1
for n in xrange(ne):
    dh = 0.029
    h = (0.27 - dh * (ne - 1)) / ne
    b = 0.89 - h * (n+1) - dh * n
    rect = (lpad+w1+d, b, w2, h)
    axes["sketch_inp_%d" % n] = fig.add_axes(rect, xticks=[], yticks=[])

rect = (lpad+w1+w2+2*d, 0.65, w3, 0.30)
axes["rec_weight"] = fig.add_axes(rect, xticks=[], yticks=[])

# inference A
loff = 0.32*wfig
md = 0.003*wfig
w1 = 0.01*wfig
w2 = 0.26*wfig
w3 = 0.05*wfig 
rect = (loff+lpad, 0.07, w1, 0.42)
axes["infA_cue"] = fig.add_axes(rect, xticks=[], yticks=[])
rect = (loff+lpad+w1+md, 0.07, w2, 0.42)
axes["infA_spike"] = fig.add_axes(rect, xticks=[], yticks=[])
rect = (loff+lpad+w1+w2+md, 0.07, w3, 0.42)
axes["infA_marginal"] = fig.add_axes(rect, xticks=[], yticks=[])

# inference B
d = 0.37*wfig
rect = (loff+lpad+d, 0.07, w1, 0.42)
axes["infB_cue"] = fig.add_axes(rect, xticks=[], yticks=[])
rect = (loff+lpad+d+w1+md, 0.07, w2, 0.42)
axes["infB_spike"] = fig.add_axes(rect, xticks=[], yticks=[])
rect = (loff+lpad+d+w1+w2+md, 0.07, w3, 0.42)
axes["infB_marginal"] = fig.add_axes(rect, xticks=[], yticks=[])

# likelihood
w = 1. - (lpad + rpad + 2*d + 0.06)
#rect = (lpad+2*d+0.06, 0.12, w, 0.37)
rect = (lpad+0.05, 0.12, w, 0.37)
axes["likelihood"] = fig.add_axes(rect)




# Overlay
rect = (0.0002, 0.0002, 0.9996, 0.9996)
axes["all"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto', frame_on=False, zorder=0)
letters = dict(A=(0.015,0.94), B=(0.39,0.94), C=(0.65,0.94), 
               D=(0.015,0.48), E=(0.31,0.48), F=(0.66,0.48))
for l,(x,y) in letters.iteritems():
    axes["all"].text(x, y, l, fontsize=8, fontweight='bold', ha='center', va='baseline')

cue_text = dict(LQ=(0.05,0.674), RQ=(0.30,0.667))
for l,(x,y) in cue_text.iteritems():
    axes["all"].text(x, y, l, fontsize=6,color='0.2', fontweight='normal', ha='center', va='baseline')


# # # # # # # # # # # # # # # 
# # # L O A D   D A T A # # # 
# # # # # # # # # # # # # # # 

print " > Load data."
fname = lambda s: "out/"+DSL+"/"+s+".npy"
archive = np.load("out/"+DSL+"/archive.npz")

for key in files:
    if np.intersect1d(files[key], SUBPLOTS).size > 0:
        print "   > Attempt to load", key, "from", fname(key),
        try:
            if key in archive.keys():
                globals()[key] = archive[key]
            else:
                globals()[key] = np.load(fname(key))
            print "[ok]."
        except Exception, e:
            print "[FAILED]."
            print "     >", repr(e)
    else:
        print "   >", key, "data not needed for selected subplots."



# post process
if "ZS" in globals():
    ZS = ZS[:,0]    # one network only
    K = ZS.shape[-1]
if "Z" in globals():
    Z = Z[:,0]    # one network only
    K = Z.shape[-1]
if "YS" in globals():
    YS = YS[:,0]    # one network only
    N = YS.shape[-1]
if "Y" in globals():
    Y = Y[:,0]    # one network only
    N = Y.shape[-1]
if "YT" in globals():
    YT = YT[:,0]    # one network only

# Rebuild old style spike array
T = int(round(max(inference_A_param["tm1"], inference_B_param["tm1"]) / inference_A_param["dt"]))
ZS = np.zeros((T,K), dtype=bool)
t, k = ZS_compressed[0][ZS_compressed[1]==0], ZS_compressed[2][ZS_compressed[1]==0]
k = k[t<T]
t = t[t<T]
ZS[t,k] = True


# # # # # # # # # # # # #
# # #   T O O L S   # # #
# # # # # # # # # # # # #

if "dims" in globals():
    gen_local = lambda n, dims: np.tile([[(k-n) % 3 == 0] for k in xrange(dims[0])], reps=(1, dims[1]))
    feat = np.array([gen_local(n, dims) for n in xrange(3)])

def get_tuning(V, V_mask):
    K = V.shape[0]
    nf = feat.shape[0]
    match = np.zeros((K,nf))
    for k in xrange(K):
        match[k] = np.dot(feat.reshape(nf,-1)[:,V_mask[k]], V[k,V_mask[k]])
    tune = match.argmax(1)
    return tune

#xy_pos = lambda k: ((k/3), 1.5*np.array([1.2,0.3,2.2,0.1,0.4,2.8,1.0])[k/3]+(k%3))
xy_pos = lambda k: ((k/3), 1.5*np.array([1.4,0.3,2.7,0.1,0.4,3.1,1.0])[k/3]+(k%3))




# # # # # # # # # # # #
# # #   P L O T   # # #
# # # # # # # # # # # #

#feat_rect = lambda n: ((9*(n%3)-0.5, 9*(n/3)-0.5), 12, 12) # (x,y),w,h

# SKETCHES
if "sketch" in SUBPLOTS:
    print " > Render sketch."
    # network sketch
    for ax,f in ([axes["sketch_net"], "fig/subplot_A.png"],):
        im = pl.imread(f)
        print "  > Image %s has dims" % f, im.shape[:2]
        ax.imshow(im, interpolation='lanczos')
        ax.set_aspect('equal')
        ax.set_frame_on(False)
    # input sketch
    sp = sketch_inp_param
    color = sp['color']
    # average sketch
    y = sp["Y"][sp['t_slice']].mean(0).reshape(dims)
    ax = axes["sketch_inp_0"]
    p = ax.get_position()
    ax.set_position([p.x0, p.y0-0.7*dh, p.width, p.width*(6./42.)*ratio])
    pl.draw()
    ax.imshow(y.T, vmin=0.0, vmax=0.5, aspect='auto', cmap=gc.input2_cmap)
    ax.set_frame_on(False)
    xs = np.arange(-0.5,dims[0],dims[1])[1:-1]
    for x in xs: ax.plot([1.005*x,1.005*x], [-1,dims[1]], c='w', lw=0.6)
    ax.set_xlim(-0.5,dims[0]-0.3)
    ax.set_ylim(-0.5,dims[1]-0.5)
    # colored sketch
    for n,ex in enumerate(sp['example']):
        ax =  axes["sketch_inp_%d" % (n+1)]
        C = np.zeros((1,7,3))
        C[:,0] = pl.matplotlib.colors.hex2color(color[ex[0]])
        C[:,1:6] = pl.matplotlib.colors.hex2color(color[ex[1]])
        C[:,6] = pl.matplotlib.colors.hex2color(color[ex[2]])
        ax.imshow(C, origin='lower', aspect='auto')
        ax.set_frame_on(False)
        # white lines
        xs = np.arange(0.5,K/3,1)
        for x in xs:
            ax.plot([1.005*x,1.005*x], [-0.5,0.5], c='w', lw=0.6)
        if n == 0:
            ax.text(0,0,"LQ",ha='center',va='center',size=6, weight='bold', color='k')
            ax.text(6,0,"RQ",ha='center',va='center',size=6, weight='bold', color='k')




# RECURRENT WEIGHTS
if "rec_weight" in SUBPLOTS:
    print " > Render rec_weight."
    ax = axes['rec_weight']
    ax.set_frame_on(False)
    grayidx = range(K)
    rwp = rec_weight_param
    hlidx = rwp['idx']
    tune = get_tuning(V, V_mask)
    color = rwp['color']
    for k in hlidx:
        grayidx.remove(k)
    x,y = xy_pos(np.array(grayidx))
    for k,yk,xk in zip(grayidx,y,x):
        c = np.array(pl.matplotlib.colors.hex2color(color[tune[k]]))
        c = 1. - 0.5 * (1. - c)
        ax.plot(xk, yk, '.', ms=10., mfc=c, mec='w', alpha=1.0, zorder=5)
    for k in hlidx:
        x,y = xy_pos(k)
        c = color[tune[k]]
        ax.plot(x, y, '.', ms=10., mfc=c, mec='w', zorder=5)
    # add cue background
    (x0,y0),(x1,y1) = xy_pos(0), xy_pos(2)
    r = pl.Rectangle((x0-0.5,y0-0.7),1.0,y1-y0+1.4,fc='0.8',ec='0.6',alpha=1.0,lw=0.3)
    ax.add_patch(r)
    ax.text(x0,y0-1.7,"LQ",ha='center',va='center',size=6, color='0.2')
    (x0,y0),(x1,y1) = xy_pos(K-3), xy_pos(K-1)
    r = pl.Rectangle((x0-0.5,y0-0.7),1.0,y1-y0+1.4,fc='0.8',ec='0.6',alpha=1.0,lw=0.3)
    ax.add_patch(r)
    ax.text(x0,y0-1.7,"RQ",ha='center',va='center',size=6, color='0.2')
    # set limits
    ax.set_xlim(-0.55,rdimsz[0]-0.45)
    ax.set_ylim(-1.,7.5)
    # lines
    lw = lambda w: (rwp['lw_max'] - rwp['lw_min']) * w/W.max() + rwp['lw_min']
    lc = lambda w: str(0.0 * (1. - lw(w) / rwp['lw_max']))
    for k in hlidx:
        for l in W_mask[k].nonzero()[0]:
            if W[k,l] < -1.:
                print "   > Skipping pair (%d,%d). Suspect inhibition." % (k,l)
                continue
            assert 0 <= W[k,l] < 5, " > ERROR with weight value!"
            x,y = xy_pos(np.array([k,l]))
            ax.plot(x, y, '-', lw=lw(W[k,l]), c=lc(W[k,l]))


# LOG LIKELIHOOD
if "likelihood" in SUBPLOTS:
    print " > Render likelihood."
    ax = axes["likelihood"]
    lp = likelihood_param
    ll = [np.load(fn) for fn in lp["ll"]]
    times = [lp["dT"] * np.load(fn) for fn in lp["times"]]
    vmin, vmax = min([min(l) for l in ll]), max([max(l) for l in ll])
    tmin, tmax = min([min(t) for t in times]), max([max(t) for t in times])
    for l,x,y,c,ls in reversed(zip(lp["label"], times, ll, lp["color"], lp["ls"])):
        line = ax.plot(x, y, c=c, label=l, ls=ls, lw=0.8)[0]
        if l in ("Wake-sleep", "W/o rec. exc."):
            line.set_dashes((2,2))
    dv = vmax - vmin
    tmax = 9600.
    ax.set_xlim(tmin, tmax)
    #ax.set_ylim(vmin-0.03*dv, vmax+0.08*dv)
    ax.set_ylim(vmin+0.3*dv, vmax+0.08*dv)
    ax.set_xticks(ax.get_xticks()[::2])
    #ax.set_xticks([0, 5000, 10000])
    ax.set_yticks(ax.get_yticks()[1::1])
    ax.xaxis.set_tick_params(length=1.5)
    ax.yaxis.set_tick_params(length=1.5)
    ax.set_xlim(tmin, tmax)
    ax.set_ylim(vmin+0.3*dv, vmax+0.08*dv)
    ax.set_xlabel("Learning time [s]", labelpad=3.)
    ax.set_ylabel("Log-likelihood", labelpad=3.)
    ax.yaxis.set_tick_params(size=1)
    ax.xaxis.set_tick_params(size=1)
    kwargs = dict(labelspacing=0.15, handlelength=1.35, borderaxespad=0.15)
    leg = ax.legend(loc='lower right', prop=dict(size=8), **kwargs)
    leg.get_frame().set_linewidth(0.5)
    leg.set_frame_on(False)
    for l in leg.get_lines():
        l.set_lw(1.0)


# UNAMBIGUOUS INFERENCE
if "inference_A" in SUBPLOTS:
    print " > Render inference_A."
    # cue
    ip = inference_A_param
    ax = axes['infA_cue']
    cue = ip['cue']
    color = ip['color']
    C = np.zeros((7,1,3))
    C[0] = pl.matplotlib.colors.hex2color(color[cue[0]])
    C[1:6] = 0.7
    C[6] = pl.matplotlib.colors.hex2color(color[cue[1]])
    ax.imshow(C, origin='lower', aspect='auto')
    ax.set_frame_on(False)
    # white lines
    ys = np.arange(0.5,K/3,1)
    for y in ys:
        ax.plot([-0.5,0.5], [y,y], c='w', lw=0.5)
    # spikes
    ax = axes['infA_spike']
    t0, t1, dt = ip['ts0'], ip['ts1'], ip['dt']
    tune = get_tuning(V, V_mask)
    cidx = np.array([(tune == i).nonzero()[0] for i in xrange(3)])
    S = ZS[int(round(t0/dt)):int(round(t1/dt))]
    ctemplate = np.ones((S.shape[0],3))
    kwargs = dict(ms=1.8, marker='|', bgcolor=ip['bgcolor'], ax=ax)
    #plot_colored_styletrain(S, t_min=0., t_max=t1-t0, dt=dt, ctemplate=ctemplate, cidx=cidx, color=color, t_scale=0.3, **kwargs)
    plot_styletrain(S, t_min=0., t_max=t1-t0, dt=dt, t_scale=0.3, **kwargs)
    ax.set_yticks([])
    #ax.spines['right'].set_visible(True)
    # white lines
    ys = np.arange(2.5,K,3)
    for y in ys:
        ax.plot([0,t1-t0], [y,y], c='w', lw=0.5, zorder=0)
    # bars
    ax = axes['infA_marginal']
    t0, t1 = int(round(ip['tm0']/dt)), int(round(ip['tm1']/dt))
    m = Z[t0:t1].mean(0)
    kwargs = dict(left=0., height=0.8, orientation="horizontal", align="center", lw=0.5, alpha=0.7)
    for i,idx in enumerate(cidx):
        c = color[i]
        bottom = idx
        width = m[idx]
        ax.bar(bottom=bottom, width=width, fc=c, **kwargs)
    ax.set_xlim(0.,1.0)
    ax.set_ylim(-0.5,K-0.5)
    ax.spines["top"].set_visible(False)
    ax.spines["left"].set_visible(True)
    ax.spines["right"].set_visible(False)
    ax.set_xticks([1])
    ax.xaxis.set_tick_params(size=1, labelsize=7)
    ax.xaxis.set_ticks_position("bottom")
    ax.set_xlabel(r"$\left< z_k \right>$", labelpad=-4.0)


# AMBIGUOS INFERENCE
if "inference_B" in SUBPLOTS:
    print " > Render inference_B."
    # cue
    ip = inference_B_param
    ax = axes['infB_cue']
    cue = ip['cue']
    color = ip['color']
    C = np.zeros((7,1,3))
    C[0] = pl.matplotlib.colors.hex2color(color[cue[0]])
    C[1:6] = 0.7
    C[6] = pl.matplotlib.colors.hex2color(color[cue[1]])
    ax.imshow(C, origin='lower', aspect='auto')
    ax.set_frame_on(False)
    # white lines
    ys = np.arange(0.5,K/3,1)
    for y in ys:
        ax.plot([-0.5,0.5], [y,y], c='w', lw=0.5)
    # spikes
    ax = axes['infB_spike']
    t0, t1, dt = ip['ts0'], ip['ts1'], ip['dt']
    tune = get_tuning(V, V_mask)
    cidx = np.array([(tune == i).nonzero()[0] for i in xrange(3)])
    S = ZS[int(round(t0/dt)):int(round(t1/dt))]
    ctemplate = np.ones((S.shape[0],3))
    kwargs = dict(ms=1.8, marker='|', bgcolor=ip['bgcolor'], show_scale=False, ax=ax)
    #plot_colored_styletrain(S, t_min=0., t_max=t1-t0, dt=dt, ctemplate=ctemplate, cidx=cidx, color=color, **kwargs)
    plot_styletrain(S, t_min=0., t_max=t1-t0, dt=dt, **kwargs)
    
    ax.set_yticks([])
    #ax.spines['right'].set_visible(True)
    # white lines
    ys = np.arange(2.5,K,3)
    for y in ys:
        ax.plot([0,t1-t0], [y,y], c='w', lw=0.5, zorder=0)
    # bars
    ax = axes['infB_marginal']
    t0, t1 = int(round(ip['tm0']/dt)), int(round(ip['tm1']/dt))
    m = Z[t0:t1].mean(0)
    kwargs = dict(left=0., height=0.8, orientation="horizontal", align="center", lw=0.5, alpha=0.7)
    for i,idx in enumerate(cidx):
        c = color[i]
        bottom = idx
        width = m[idx]
        ax.bar(bottom=bottom, width=width, fc=c, **kwargs)
    ax.set_xlim(0.,1.0)
    ax.set_ylim(-0.5,K-0.5)
    ax.spines["top"].set_visible(False)
    ax.spines["left"].set_visible(True)
    ax.spines["right"].set_visible(False)
    ax.set_xticks([1])
    ax.xaxis.set_tick_params(size=1, labelsize=7)
    ax.xaxis.set_ticks_position("bottom")
    ax.set_xlabel(r"$\left< z_k \right>$", labelpad=-4.0)


# # # # # # # # # # # # # #
# # # F I N A L I Z E # # #
# # # # # # # # # # # # # #
pl.interactive(True)

print " > Show figure."
pl.show()
pl.draw()
print " > Done."

if SAVE is True:
    print " > Save figure to 'fig/Figure.{png,pdf}'."
    fig.savefig("fig/Figure.png")
    fig.savefig("fig/Figure.pdf")

