import numpy as np
import pylab as pl

import sys
sys.path.append('../')
import my_matplotlib_defaults

from liveplot import LivePlot
import time
import gwta_shared as gwta
from sys import stdout

# # # INIT (extract and save params) # # #
paramfilename = gwta.get_param_file()
print " > Loading parameters from file " + paramfilename + ".py"
exec("from " + paramfilename + " import param")     # load parameter dictionary
gwta.extract_params(param, globals(), keep=('samplerKwargs', 'kwargs_poszk', 'kwargs_posyi', 'kwargs_connV', 'kwargs_connW', 'inputKwargs', 'pspKwargs', 'VKwargs', 'kwargs_rec_plast'))

print " > DSL is", DSL
print " > Simulation will" + (spiking and ' ' or ' NOT ') + "be spiking."

# set random seed
np.random.seed(seed)

# create logger for sparsely saved data
kwargs = dict(Wtrace=("W", (K,K), 'float'), btrace=("b", (K,), 'float'), Vtrace=("V", (K,N), 'float'))
if PRIOR:
    kwargs["b_priortrace"] = ("b_prior", (K,), 'float')
logger = gwta.Logger(DSL, T, SAVEEVERY, TEST, globals(), **kwargs)
logger.save_param(paramfilename)


# # # GENERATE NETWORK # # #

# Build network structure
net = gwta.NetworkBuilder(K, N, dims, pbc=topo_pbc)
rdimsz = net.set_pos_raster("zk", **kwargs_poszk)
rdimsy = net.set_pos_raster("yi", **kwargs_posyi)
net.set_V(**kwargs_connV)
net.set_W(**kwargs_connW)
subnet = net.add_subnet(**kwargs_connW)
net.check()

# neuron and synapse arrays
V = VsynapseClass(K=K, N=N, mask=net.V_mask, **VKwargs)     # input synapse object
W = gwta.shmarray.zeros((K,K), dtype='d')                   # shared sampler matrix
b = gwta.shmarray.zeros(K, dtype='d')                       # shared sampler vector
V0 = V.get_V0()
ext = np.zeros((num_parallel,K))                            # vector for summed external input
V[:] = net.V
V_mask = net.V_mask
W[:] = net.W
W_mask = net.W_mask
b[:] = net.b


# load synapses and biases from file
if load_V_from_file:
    print " > Loading V from file '%s'." % load_V_from_file
    V.W[:] = np.load(load_V_from_file)

if load_b_from_file:
    print " > Loading b from file '%s'." % load_b_from_file
    b[:] = np.load(load_b_from_file)

if load_W_from_file:
    print " > Loading W from file '%s'." % load_W_from_file
    W_mask_fname = load_W_from_file[:-4] + "_mask.npy"
    W_mask[:] = np.load(W_mask_fname)
    W[W_mask] = np.load(load_W_from_file)[W_mask]
    # Clipping only occurs when loading from file
    if kwargs_rec_plast.has_key("Wmax") and (PRIOR is False) and (eta_W > 0.) and (eta_scale(0) > 0.):
        print "  > Clipping to [-inf, %.3f]" % (kwargs_rec_plast["Wmax"] - 0.01)
        np.clip(W, -100000., kwargs_rec_plast["Wmax"] - 0.01, out=W)



# init parallel processes
sampler = samplerClass(W, b, **samplerKwargs)
co = gwta.Coordinator(sampler, num_parallel, spiking)
num_parallel = co.num_parallel

# Prior samplers
assert isinstance(PRIOR, int), " > ERROR: PRIOR must be False or int!"
if PRIOR is not False:
    print " > PRIOR samplers will be available in this simulation."
    assert rec_plast_rule is gwta.wake_sleep_plasticity, " > ERROR: If PRIOR, use gwta.wake_sleep_plasticity for recurrent connections!"
    b_prior = gwta.shmarray.zeros(K, dtype='d')         # shared sampler vector
    if load_b_prior_from_file:
        print " > Loading b_prior from file '%s'." % load_b_prior_from_file
        b_prior[:] = np.load(load_b_prior_from_file)
    else:
        b_prior[:] = b + V.get_Ak()                         # estimate prior
    sampler_priorKwargs = dict()
    sampler_priorKwargs.update(samplerKwargs)
    sampler_priorKwargs.pop('tau',None)                 # Remove 'tau' is present
    sampler_priorKwargs['burnin'] = PRIOR               # Set/Add 'burnin'
    sampler_prior = gwta.SamplerGibbs(W, b_prior, **sampler_priorKwargs)
    co_prior = gwta.Coordinator(sampler_prior, num_parallel, spiking=False)



# # # MEMORY CHECK # # #
# memory check for full array (Z, Y, shownTemplate, maybe: spikes)
tmpa = [((num_parallel,K),"int8"),((num_parallel,N),"int"),((num_parallel,),"int")]
if spiking: tmpa += [((num_parallel,K),"int8"), ((num_parallel,N),"int")]
if PRIOR: tmpa += [((num_parallel,K),"int8"),]
gwta.memorycheck(T=T, arrays=tmpa, limit=0.9)




# # # GENERATE INPUT # # #
if NOINPUT:
    print " > WARNING: Input is switched off! Only zeros are presented!"
    try:
        template = inputKwargs['template']
    except:
        print " > Warning: No input templates defined."
    Y = np.zeros((T,num_parallel,N), dtype=np.float16)
    YT = Y
    if spiking:
        YS = Y
else:
    print " > Generating input of shape (%d,%d,%d)" % (T, num_parallel, N)
    try:
        template = inputKwargs['template']
    except:
        print " > Warning: No input templates defined."
    inpgen = InputClass(num_parallel=num_parallel, spiking=spiking, tau=tau, **inputKwargs)
    if 'inputSequence' not in locals():
        inputSequence = None
    if spiking:
        #print "WARNING: HACK!"
        #seq = np.array(([7]*500 + [112]*500 + [71]*500)*40).reshape(60000,1)
        #YT, YS = inpgen.generate(T, sequence=seq)
        YT, YS = inpgen.generate(T, inputSequence)
        print " > Applying PSP kernel."
        Y = gwta.applyFastBinaryKernel(YS, tau, outdtype=np.float16)
        #Y = gwta.applyPSPkernel(S=YS, dtype=np.float16, **pspKwargs)
    else:
        YT, Y = inpgen.generate(T, inputSequence)


# Adapt V0
if setV0_inpmean:
    print " > Setting V0 to match input marginals."
    V.set_V0(V._logit(Y[:100000].mean(1).astype(float).mean(0)))
    V0 = V.get_V0()


# # # RESULT ARRAYS # # #
Z = np.zeros((T,num_parallel,K), dtype=np.int8)
if spiking:
    ZS = np.zeros((T,num_parallel,K), dtype=np.int8)
if PRIOR:
    Z_prior = np.zeros((T,num_parallel,K), dtype=np.int8)

# # # START SIM # # #
# start samplers
co.start_samplers()
if PRIOR: co_prior.start_samplers()

# last preparations
if PLOTEVERY:
    plotter = LivePlot(sim_globals=globals())
ETA = lambda t: t and ( (float(T) / t - 1.) * (time.time() - time_start) )
reportmodulo = int(round(1./dt))
pe = gwta.ParameterEvolution(Nfilter=10, V=V.W, b=b, W=W)
print "\n > Start simulation of %.1fs (%d samples per sampler)" % (T_si, T)
time_start = time.time()


# # # MAIN LOOP # # #
for t in xrange(T):
    if t % reportmodulo == 0:
        print " > Time t=%ds \t ETA %.1fs" % (int(round(t*dt)), ETA(t))
        print "  > Mean(V) = %.2e" % (V[V_mask].mean())
        print "  > Mean(W) = %.2e" % (W[W_mask].mean())
        pe.get_stats(V=V.W, b=b, W=W)
    if PLOTEVERY and t % PLOTEVERY == 0:
        print " > plotting...",; stdout.flush()
        plotter.update()
        #recplotter.update()
        print "Done."; stdout.flush()
    # distribute input
    for i in xrange(num_parallel):
        ext[i] = V.transmit(Y[t,i])
    co.distribute_ext(ext)
    # unleash samplers
    co.sampling_initiate()
    if PRIOR: co_prior.sampling_initiate()
    # learning on previous states
    if Tplast[0] < t <= Tplast[1]:
        if eta_V > 0.:
            V.plasticity(eta_scale(t)*eta_V, Z[t-1], Y[t-1])
        if eta_b > 0.:
            b += eta_scale(t) * eta_b * num_parallel * (m_hom - Z[t-1].mean(0))
        if eta_W > 0.:
            if PRIOR:
                db_prior, dW = rec_plast_rule(Z[t-1], Z_prior[t-1])
            else:
                #dW = rec_plast_rule(W, Z[t-1], m_hom, alpha=4.5, lam=2)
                dW = rec_plast_rule(W, Z[t-1], m_hom, **kwargs_rec_plast)
            W += eta_scale(t) * eta_W * W_mask * dW
            if PRIOR:
                b_prior += eta_scale(t) * eta_b * db_prior
            if only_exc_W:
                W[:] = (1-W_mask) * W + W_mask * np.clip(W, 0., 10000.)
    # collect samples
    if spiking:
        Z[t], ZS[t] = co.sampling_collect()
    else:
        Z[t] = co.sampling_collect()
    if PRIOR:
        Z_prior[t] = co_prior.sampling_collect()
    # track results
    logger.track(t)


# # # FINALIZE AND SAVE RESULTS # # #
print " > Simulation took %.1fs" % (time.time() - time_start)
if PLOTEVERY:
    plotter.update()
    #recplotter.update()

co.finalize()
if PRIOR: co_prior.finalize()

centers = V.get_centers()
Ak = V.get_Ak()

if PRIOR:  # fit parameters of heuristic learning rule
    from scipy.optimize import curve_fit
    zcov = Z[-1000000:,0]
    c = np.cov(zcov.T)
    idx = (W > 0.01) * W_mask
    c_max = c[idx].max()
    f = lambda c, Wmax, gamma: Wmax/(np.pi/2.) * np.arctan(gamma * c)
    (Wmax, gamma),_ = curve_fit(f, c[idx], W[idx], p0=(1.,10.))
    print "Wmax = %.4f \t gamma =%.4f" % (Wmax, gamma)
    fit_params = (Wmax, gamma)

kwargs = dict(individual = ["subnet", "dims", "rdimsz", "rdimsy", "V0",
                            "V", "V_mask", "W", "W_mask", "b", "centers", "Ak"],
              archive = [])

if globals().has_key("template"):
    kwargs['individual'] += ["template"]

# Check size limit:
SIZELIMIT_GB = 2.
limited_list = ["Y", "YT", "Z", "Wtrace", "btrace", "Vtrace"]
for key in limited_list:
    if globals().has_key(key):
        x = globals()[key]
    else:
        x = logger.__getattribute__(key)
    if x.nbytes / 2.**30 < SIZELIMIT_GB:
        kwargs['archive'] += [key]
    else:
        print " > Warning: excluding '%s' from archive (larger than %.1f GB)." % (key, SIZELIMIT_GB)


if spiking:
    print " > Compress input spikes."
    t, p, i = YS.nonzero()
    n = YS[(t,p,i)]
    YS_compressed = np.vstack((t,p,i,n)).astype(np.int32)
    kwargs['archive'].append("YS_compressed")
    print " > Compress network spikes."
    t, p, i = ZS.nonzero()
    n = ZS[(t,p,i)]
    ZS_compressed = np.vstack((t,p,i,n)).astype(np.int32)
    kwargs['archive'].append("ZS_compressed")


if PRIOR:
    kwargs['individual'] += ["Z_prior", "b_prior", "fit_params"]
    kwargs['archive'] += ["b_priortrace",]

logger.save(**kwargs)





