import numpy as np
import time
import sys
sys.path.append('../')
import my_matplotlib_defaults
import gwta_shared as gwta

import time
from billtool import generate_DSL

# # # # # # # # # # # # # # # # # # # # # # # #
# # # G E N E R A T I N G   P A R A M S # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

np.random.seed(0)       # seed for parameter generation
DSL = "Fig5_train_ws"           # Data Set Locator defines the directory name where
                            # results are stored within the "out/"-directory
#DSL = generate_DSL()        # Use this to generate a timestamp-DSL
#                            # If used, the DSL has to be changed in other scripts

# Original paper simulation time and learning rate
T_si = 10000. / 5              # Total time in SI units
eta_scale_fun = lambda t: 0.2/tau
# Quick test simulation time and learning rate
#T_si = 10000. / 100                # Total time in SI units
#eta_scale_fun = lambda t: 10.0/tau

dt = 0.001                   # discretization time step in SI units
tau_si = 0.010               # tau in SI units

K = 12*12 # 20**2 - 20
N = 24*24 # 20**2
dims = (24,24) # (20,20)
connradius_V = 2.6 # 2.6
num_subnet = 1

THEOBASED = True # = wake-sleep
LOAD_DSL = "Fig5_train_ws1"

rec_plast_rule = THEOBASED and gwta.wake_sleep_plasticity or gwta.covariance_plasticity_tan

template = np.load("data/windows_24x24.npy")
template = template.reshape(template.shape[0], -1)
template = 0.1 + (0.5 - 0.1) * template


# # # # # # # # # # # # # # # # # # # # # # # #
# # # # # P R E P R O C E S S I N G # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

tau = int(round(tau_si/dt))
T = int(round(T_si/dt))
load_fname = lambda s: LOAD_DSL and "./out/"+LOAD_DSL+"/"+s+".npy" or False



# # # # # # # # # # # # # # # # # # # # # # # #
# # # # # A C T U A L   P A R A M S # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # #

param = {
    "global" : dict(
        T_si = T_si,
        T = T,
        dt = dt,
        seed = None,
        DSL = DSL,
        num_parallel = 5,           # level of parallelization during learning (int or 'auto')
        spiking = True,             # whether to use spikes streams or independent samples
        Tplast = (0, T),            # plasticity will be switched on/off at these times 
        PRIOR = THEOBASED,                  # False or burnin; Also generate num_parallel independent Gibbs prior samplers?
        TEST = False,                # whether to save params and results
        PLOTEVERY = 10000,          # Plotter update, timesteps or None/0
        SAVEEVERY = 1000,            # save only every n-th step (for logger)
         ),
    "input" : dict(
         NOINPUT = False,            # present zeros only
         InputClass = gwta.FromTemplateBernoulliGenerator,
         inputKwargs = {"template" : template, "duration" : int(round(0.100/dt))},
         #InputClass = gwta.FromTemplateBernoulliGenerator,
         #inputKwargs = {'duration' : int(round(0.500/dt)), 'template' : template},
         #pspKwargs = {'kernel' : [1.]*tau, 'postprocess' : lambda y : np.clip(y, 0, 1)},    # rectangular, renewing PSPs
         ),
    "net" : dict(
        dims = dims,
        K = K,
        N = N,
        tau = tau,
        kwargs_poszk = {"dist" : (2,2), "offset" : (0.5,0.5)},
        kwargs_posyi = {"dist" : (1,1), "offset" : (0,0)},
        kwargs_connV = {"connradius" : connradius_V, "norm" : "max-norm", "p_plast" : 1.0, "Vinit" : 0.0},
        kwargs_connW = {"connradius" : 100., "p_plast" : 0.25, "num_net" : num_subnet, "w" : 0., 
                        "norm" : "max-norm", "b_init" : -3., "W_inh" : -100.},
        VsynapseClass = gwta.BernoulliSynapses,
        VKwargs = {'syninit' : lambda k,i: 0., 'pi0i' : np.ones(N) * 0.1,
                                                            #np.ones(template.shape[1]) * template.min(),
                                                            # template.mean(0),
                                                            # np.ones(template.shape[1]) * template.min(),
                   'semlike' : False, 'calcAk' : False, 'only_exc' : True},
        setV0_inpmean = False,
        load_b_prior_from_file = load_fname("b_prior"),
        load_b_from_file = load_fname("b"),
        load_V_from_file = load_fname("V"),
        load_W_from_file = load_fname("W"),
        samplerClass =  gwta.SamplerAbsref,
        samplerKwargs =  {'tau': tau, 'shuffleIdx': True },
        m_hom = 0.025,
        ),
    "train" : dict(
        eta_V = 0.02,
        eta_b = 0.1,
        eta_W = 0.01, # 0.001,
        rec_plast_rule = rec_plast_rule,
        kwargs_rec_plast ={"Wmax": 2.7049, "gamma" : 733.69},
        only_exc_W = True,
        eta_scale = eta_scale_fun,  
        ),
}




