# Using the magic encoding
# -*- coding: utf-8 -*-

import numpy as np
import pylab as pl
import sys
sys.path.append('../')
from figure_format_plos import fig_width, cm2inch
from gwta_shared.plotting import plot_styletrain
from gwta_shared.builder import dist_pbc
from gwta_shared import color as gc


# # # # # # # # # # # # 
# # # P A R A M S # # # 
# # # # # # # # # # # # 

ZOOM = 1.5
SUBPLOTS = ("sketch", "W_line", "V", "W_group", "W_dist", "ZS", "YS", "explain", "y_example")
SAVE = True

DSL = "Fig5_eval" 
feat = np.array([[(i%3==0) or (j%3==0) for i in range(24) for j in range(24)],
                 [(i+j)/2 % 2 for i in range(24) for j in range(24)],
                 [(i+j) % 2 for i in range(24) for j in range(24)]]).reshape(-1,24,24)




files = dict(ZS_compressed=("ZS",),    # Files required by different subplots: DSL_KEY.npy=("A", "B")
             Z = ("SCC","explain"),
             Y = ("y_example","explain","YS"),
             YS_compressed=("YS",),
             YT=("SCC","y_example","W_group"),
             W=("W_line", "W_group","W_dist"),
             W_mask=("W_group","W_dist"),
             V = ("V","W_line","W_group","explain"),
             V_mask = ("V","W_line","W_group","explain"),
             rdimsz=("W_line", "SCC","W_dist","explain"),
             dims=("y_example","explain"),
             V0=("explain",),
            )

ZS_param = dict(t_min=23.250, t_max=25.250, dt=0.001, bgcolor = gc.network_spikebg)
YS_param = dict(Nplot=144, t_min=23.250, t_max=25.250, dt=0.001, bgcolor = gc.input_spikebg)


idx = np.ravel_multi_index(np.transpose([(1,6), (2,3), (2,9), (5,10),
                                         (9,3), (7,10), (10,6), (5,2)]), (12,12))

W_line_param = dict(idx=idx, lw_max=1., 
                    color=(gc.select['red'], gc.select['green'], gc.select['blue']))
W_dist_param = dict(local_dist=4.5)
V_param = dict(neuron={},)
y_example_param = dict(t=(30000,27000,33000), t_inst=100, t_mean=500)
explain_param = dict(tidx=[range(23500,24000,1), range(24500,25000,1)], back_vmin=0.0, back_vmax=1.0, dt=0.001)

# # # # # # # # # # #
# # # S E T U P # # # 
# # # # # # # # # # # 

print " > DSL =", DSL
print " > Subplots:", SUBPLOTS

# Zooming
pl.rc("figure", dpi=ZOOM*pl.rcParams['figure.dpi'])

# Figure setup
fig = pl.figure(figsize=(fig_width['two_col'], cm2inch(13.)))

axes = dict()

lpad, rpad = 0.07, 0.05
wfig = 1. - lpad - rpad
ratio = fig.get_figwidth() / fig.get_figheight()

# add axes
rect = (0.0001, 0.0001, 0.9998, 0.9998)
axes["all"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='auto',
                           xlim=(0,1), ylim=(0,1), frame_on=False, zorder=0, )

w,h = 0.45*wfig, 0.28
print " > Size of axes 'sketch': %d x %d pixels." % \
         (w*fig.get_figwidth()*rcParams['savefig.dpi'], h*fig.get_figheight()*rcParams['savefig.dpi'])
rect = (lpad, 0.70, w, h)
axes["sketch"] = fig.add_axes(rect, xticks=[], yticks=[])

rect = (lpad, 0.39, 0.33*wfig, 0.23)
axes["ZS"] = fig.add_axes(rect)

rect = (lpad, 0.12, 0.33*wfig, 0.23)
axes["YS"] = fig.add_axes(rect)


w = 0.07
d = 0.05
h = ratio * w
for i in xrange(3):
    l = lpad + 0.15 + 0.45*wfig + i*(d+w)
    rect = (l, 0.87, w, h)
    axes["ym%d" % i] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal')
    a = pl.Arrow(l+0.5*w, 0.865, 0., -0.025, width=0.011, ec='k', lw=0., fc='k', fill=True, alpha=0.7)
    axes['all'].add_patch(a)

for i in xrange(3):
    l = lpad + 0.15 + 0.45*wfig + i*(d+w)
    rect = (l, 0.74, w, h)
    axes["y%d" % i] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal')


# EXPLAIN INPUT
# LEFT
l = lpad + 0.33*wfig + 0.03
w = 0.07
h = ratio * w
m = 0.37
b = m+h/2+0.07
rect = (l, b, w, h)
axes["expl_zmean_0"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal')
b = m-h/2
rect = (l, b, w, h)
axes["expl_ynet_0"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal')
b = m-3*h/2-0.07
rect = (l, b, w, h)
axes["expl_ydata_0"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal')
b = m+h/2+0.07
a = pl.Arrow(l+0.5*w, b-0.005, 0., -0.06, width=0.011, ec='k', lw=0., fc='k', fill=True, alpha=0.7)
axes['all'].add_patch(a)
xy = l-0.01, 0.12
patch = pl.Rectangle(xy, 0.09, 0.50, fc='0.97', ec='0.55', lw=0.5, zorder=0)
axes['all'].add_patch(patch)

FAP = pl.matplotlib.patches.FancyArrowPatch
kwargs = dict(lw=0.5, color='0.55', mutation_scale=60,
              connectionstyle="angle3,angleA=32,angleB=-32",
              arrowstyle="-|>,head_length=0.4,head_width=0.2")

posA, posB=(0.141,0.618), (l+0.040,0.622)
arrow = FAP(posA, posB,**kwargs)
axes['all'].add_patch(arrow)
posA, posB=(0.141,0.122), (l+0.040,0.118)
arrow = FAP(posA, posB,**kwargs)
arrow.set_connectionstyle("angle3,angleA=-20,angleB=20")
axes['all'].add_patch(arrow)


# RIGHT
l = lpad + 0.33*wfig + 0.03 + 0.10
b = m+h/2+0.07
rect = (l, b, w, h)
axes["expl_zmean_1"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal')
b = m-h/2
rect = (l, b, w, h)
axes["expl_ynet_1"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal')
b = m-3*h/2-0.07
rect = (l, b, w, h)
axes["expl_ydata_1"] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal')
b = m+h/2+0.07
a = pl.Arrow(l+0.5*w, b-0.005, 0., -0.06, width=0.011, ec='k', lw=0., fc='k', fill=True, alpha=0.7)
axes['all'].add_patch(a)
xy = l-0.01, 0.12
patch = pl.Rectangle(xy, 0.09, 0.50, fc='0.97', ec='0.55', lw=0.5, zorder=0)
axes['all'].add_patch(patch)

posA, posB=(0.287,0.618), (l+0.035,0.622)
arrow = FAP(posA, posB,**kwargs)
arrow.set_connectionstyle("angle3,angleA=35,angleB=-35")
axes['all'].add_patch(arrow)
posA, posB=(0.287,0.122), (l+0.035,0.118)
arrow = FAP(posA, posB,**kwargs)
arrow.set_connectionstyle("angle3,angleA=-24,angleB=24")
axes['all'].add_patch(arrow)

l = lpad + 0.2 + 0.45*wfig
rect = (l, 0.32+0.05, 0.25*wfig, 0.30*wfig)
axes["W_line"] = fig.add_axes(rect, xticks=[], yticks=[], xlim=(-0.5,11.5), ylim=(-0.5,11.5), aspect='equal', frame_on=False)

# calc postions of highlighted neuron axes
def calc_V_ax_lb(k, dims):
    c = (np.array(dims, dtype=float) - 1.) / 2.   # center
    p = np.unravel_index([k], dims)
    p = np.array([p[0][0], p[1][0]])
    v = (p-c) / np.linalg.norm(p-c)
    # transformatio party
    trans = axes["W_line"].transData.transform
    inv = fig.transFigure.inverted().transform
    l,b = inv(trans(p[::-1])) + 0.035 * v[::-1] * (1., 1.5)
    return l,b

for k in W_line_param['idx']:
    V_param['neuron'][k] = calc_V_ax_lb(k, dims=(12,12))

for k,(l,b) in V_param['neuron'].iteritems():
    rect = (l-0.025, b-0.025, 0.05, 0.05)
    axes["V%d" % k] = fig.add_axes(rect, xticks=[], yticks=[], aspect='equal', frame_on=True)


l = 1-rpad-2*0.11*wfig-0.08
rect = (l, 0.12, 0.11*wfig, 0.15)
axes["W_group"] = fig.add_axes(rect)

l = 1-rpad-0.11*wfig
rect = (l, 0.12, 0.11*wfig, 0.15)
axes["W_dist"] = fig.add_axes(rect)



# Overlay
letters = dict(A=(0.02,0.97), B=(0.56,0.97),
               C=(0.02,0.635), D=(0.37,0.635), E=(0.63,0.67),
                                             F=(0.61,0.29))
for l,(x,y) in letters.iteritems():
    axes["all"].text(x, y, l, fontsize=8, fontweight='bold', ha='center', va='baseline')

# Lines


# # # # # # # # # # # # # # # 
# # # L O A D   D A T A # # # 
# # # # # # # # # # # # # # # 

print " > Load data."
fname = lambda s: "out/"+DSL+"/"+s+".npy"
archive = np.load("out/"+DSL+"/archive.npz")
for key in files:
    if np.intersect1d(files[key], SUBPLOTS).size > 0:
        print "   > Attempt to load", key, "from", fname(key),
        try:
            if key in archive.keys():
                globals()[key] = archive[key]
            else:
                globals()[key] = np.load(fname(key))
            print "[ok]."
        except Exception, e:
            print "[FAILED]."
            print "     >", repr(e)
    else:
        print "   >", key, "data not needed for selected subplots."



# post process
if "ZS" in globals():
    ZS = ZS[:,0]    # one network only
    K = ZS.shape[-1]
if "Z" in globals():
    Z = Z[:,0]    # one network only
    K = Z.shape[-1]
if "YS" in globals():
    YS = YS[:,0]    # one network only
    N = YS.shape[-1]
if "Y" in globals():
    Y = Y[:,0]    # one network only
    N = Y.shape[-1]
if "YT" in globals():
    YT = YT[:,0]    # one network only



# # # # # # # # # # # # #
# # #   T O O L S   # # #
# # # # # # # # # # # # #

def get_tuning(V, V_mask, feat):
    K = V.shape[0]
    nf = feat.shape[0]
    match = np.zeros((K,nf))
    for k in xrange(K):
        match[k] = np.dot(feat.reshape(nf,-1)[:,V_mask[k]], V[k,V_mask[k]])
    tune = match.argmax(1)
    return tune





# # # # # # # # # # # #
# # #   P L O T   # # #
# # # # # # # # # # # #

feat_rect = lambda n: ((9*(n%3)-0.5, 9*(n/3)-0.5), 12, 12) # (x,y),w,h

if "sketch" in SUBPLOTS:
    print " > Render sketch."
    for ax,f in ([axes["sketch"], "fig/sketch.png"],):
        im = pl.imread(f)
        print "  > Image %s has dims" % f, im.shape[:2][::-1]
        ax.imshow(im, interpolation='lanczos')
        ax.set_frame_on(False)




if "y_example" in SUBPLOTS:
    print " > Render y_example."
    # means
    for i in xrange(3):
        ax = axes['ym%d' % i]
        t = y_example_param['t'][i]
        t_mean = y_example_param['t_mean']
        ax.imshow(Y[t:t+t_mean].mean(0).reshape(dims), cmap=gc.input_cmap, vmin=0, vmax=1, origin='lower')
    # instances
    for i in xrange(3):
        ax = axes['y%d' % i]
        t = y_example_param['t'][i] + y_example_param['t_inst']
        ax.imshow(Y[t].reshape(dims), cmap=gc.input_cmap, vmin=0, vmax=1, origin='lower')




# SPIKE COUNT CORRELATION
if "SCC" in SUBPLOTS:
    print " > Render SCC."
    ax = axes['SCC']
    kidx = np.arange(100).reshape(rdimsz)[3:7,3:7].flatten() 
    nondiagidx = True - np.diag([True]*len(kidx))
    # noise correlation long window
    t_mean = 500
    rsc = []
    cc_rsc = []
    for i in xrange(0,3):
        z = Z[(TC[YT] == i)*(TF[YT,4] == i+1)][:,kidx].reshape(-1, t_mean, len(kidx)).mean(1)
        cc = np.corrcoef(z.T)
        cc_rsc.append(cc)
        rsc.append(cc[nondiagidx])
    rsc = np.mean(rsc,0)
    # noise correlation short window
    t_mean = 1
    rsc_local = []
    cc_rsc_local = []
    for i in xrange(0,3):
        z = Z[(TC[YT] == i)*(TF[YT,4] == i+1)][:,kidx].reshape(-1, t_mean, len(kidx)).mean(1)
        cc = np.corrcoef(z.T)
        cc_rsc_local.append(cc)
        rsc_local.append(cc[nondiagidx])
    rsc_local = np.mean(rsc_local,0)
    # signal correlation
    m = []
    for i in xrange(0,3):
        z = Z[(TC[YT] == i)*(TF[YT,4] == i+1)][:,kidx].mean(0)
        m.append(z.copy())
    m = np.array(m)
    cc = np.corrcoef(m.T)
    rsignal = cc[nondiagidx]
    # plot
    ax.plot(rsignal, rsc, 'b.', label="Long")
    ax.plot(rsignal, rsc_local, 'r.', label="Short")
    leg = ax.legend(loc='upper left')



# WEIGHTS
if "W_line" in SUBPLOTS:
    print " > Render W_line."
    ax = axes['W_line']
    K = np.prod(rdimsz)
    # dots
    grayidx = range(K)
    hlidx = W_line_param['idx']
    tune = get_tuning(V, V_mask, feat)
    color = W_line_param['color']
    for k in hlidx:
        grayidx.remove(k)
    y,x = np.unravel_index(grayidx, rdimsz)
    for k,yk,xk in zip(grayidx,y,x):
        c = color[tune[k]]
        ax.plot(xk, yk, '.', ms=13., mfc=c, mec='w', alpha=0.3)
    for k in hlidx:
        y,x = np.unravel_index(k, rdimsz)
        c = color[tune[k]]
        ax.plot(x, y, '.', ms=13., mfc=c, mec='w', zorder=5)
    # lines
    for k in hlidx:
        for l in hlidx:
            if k >= l: continue
            if W[k,l] < -1.:
                print "   > Skipping pair (%d,%d). Suspect inhibition." % (k,l)
                continue
            assert 0 <= W[k,l] < 10, " > ERROR with weight value!"
            lw = W_line_param['lw_max'] * W[k,l]
            y,x = np.unravel_index([k,l], rdimsz)
            ax.plot(x, y, 'k-', lw=lw)
    ax.set_xlim(-0.5, rdimsz[1]-0.5)
    ax.set_ylim(-0.5, rdimsz[0]-0.5)

if "V" in SUBPLOTS:
    print " > Render V."
    vmax = V.max()
    for k in V_param['neuron']:
        ax = axes["V%d" % k]
        rfe = int(np.sqrt(V_mask[0].sum()))
        Vlocal = V[k][V_mask[k]].reshape(rfe,rfe)
        ax.imshow(Vlocal, vmin=0, vmax=vmax, cmap=gc.weight_cmap, origin='lower')
        ax.set_xlim(-0.5,rfe-0.5)
        ax.set_ylim(-0.5,rfe-0.5)


if "W_group" in SUBPLOTS:
    print " > Render W_group."
    ax = axes['W_group']
    # find tuning
    tune = get_tuning(V, V_mask, feat)
    w_intra = W[W_mask * (np.outer(tune==0,tune==0) + np.outer(tune==1,tune==1) + np.outer(tune==2,tune==2))]
    w_inter = W[W_mask * (np.outer(tune==0,tune==1) + np.outer(tune==0,tune==2) + np.outer(tune==1,tune==0)
                         + np.outer(tune==1,tune==2) + np.outer(tune==2,tune==0) + np.outer(tune==2,tune==1))]
    l = 0.1,1.1
    bottom = -1.
    h = w_intra.mean()-bottom, w_inter.mean()-bottom
    ax.bar(l, h, bottom=bottom, fc='0.7', lw=0.5)
    ax.set_xlim(-0.1,2.1)
    ax.set_xticks([])
    ax.set_yticks([0.,0.25,0.5])
    ax.set_ylim(0,0.65)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')
    ax.set_ylabel("Avg. weight")
    ax.set_xlabel(u'\u2022\u2212\u2022  \u2022\u2212\u25E6', size=12)


if "W_dist" in SUBPLOTS:
    print " > Render W_dist."
    ax = axes['W_dist']
    K = W.shape[0]
    # collect distance and weight
    w = []
    for k in xrange(K):
        x = np.unravel_index(k, rdimsz)
        for l in range(k):
            if W_mask[k,l]:
                y = np.unravel_index(l, rdimsz)
                r = dist_pbc(x,y,rdimsz)
                w.append([r, W[k,l]])
    w = np.array(w)
    idx = w[:,0] <= W_dist_param['local_dist']
    w_local = w[idx,1]
    w_long = w[np.invert(idx),1]
    # plot
    l = 0.1,1.1
    bottom = -1.
    h = w_local.mean()-bottom, w_long.mean()-bottom
    ax.bar(l, h, bottom=bottom, fc='0.7', lw=0.5)
    ax.set_xlim(-0.1,2.1)
    ax.set_xticks([])
    ax.set_yticks([0.,0.25, 0.5])
    ax.set_ylim(0,0.65)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')
    ax.set_ylabel("Avg. weight")
    ax.set_xlabel(u"\u2264%.1f   >%.1f" % (W_dist_param['local_dist'], W_dist_param['local_dist']))


# SPIKE TRAINS
if "ZS" in SUBPLOTS:
    print " > Render ZS."
    ax = axes['ZS']
    # Rebuild old style spike array
    T = int(ZS_param['t_max'] / ZS_param['dt']) + 1
    ZS = np.zeros((T,K), dtype=bool)
    t, k = ZS_compressed[0][ZS_compressed[1]==0], ZS_compressed[2][ZS_compressed[1]==0]
    k = k[t<T]
    t = t[t<T]
    ZS[t,k] = True
    plot_styletrain(ZS, t_min=ZS_param['t_min'], t_max=ZS_param['t_max'], bgcolor=ZS_param['bgcolor'], show_scale=False, t_scale=0.3, ax=ax)
    ax.set_ylabel("Network neurons", labelpad=0.)
    # gray shades
    dt = explain_param['dt']
    for r in explain_param['tidx']:
        x,y = min(r) * dt, 0
        w,h = (max(r) - min(r)) * dt, K
        p = pl.Rectangle((x,y), w, h, fc="k", ec='k', lw=0, alpha=0.10, zorder=5)
        ax.add_patch(p)


if "YS" in SUBPLOTS:
    print " > Render YS."
    ax = axes['YS']
    # Rebuild old style spike array
    T = int(YS_param['t_max'] / YS_param['dt']) + 1
    YS = np.zeros((T,N), dtype=bool)
    t, k = YS_compressed[0][YS_compressed[1]==0], YS_compressed[2][YS_compressed[1]==0]
    k = k[t<T]
    t = t[t<T]
    YS[t,k] = True
    idx = np.arange(N)
    np.random.shuffle(idx)
    idx = idx[:YS_param['Nplot']]
    plot_styletrain(YS[:,idx], t_min=YS_param['t_min'], t_max=YS_param['t_max'], bgcolor=YS_param['bgcolor'], t_scale=0.3, ax=ax)
    ax.set_ylabel("Input neurons", labelpad=0.)
    # gray shades
    dt = explain_param['dt']
    for r in explain_param['tidx']:
        x,y = min(r) * dt, 0
        w,h = (max(r) - min(r)) * dt, K
        p = pl.Rectangle((x,y), w, h, fc="k", ec='k', lw=0, alpha=0.10, zorder=5)
        ax.add_patch(p)




# EXPLANATION
if "explain" in SUBPLOTS:
    print " > Render explanation."
    for i,tidx in enumerate(explain_param['tidx']):
        # mean response
        ax = axes['expl_zmean_%d' % i]
        z = Z[tidx].mean(0).reshape(rdimsz)
        ax.imshow(z, vmin=0, vmax=1, cmap=gc.network_cmap, origin='lower')
        # explanation
        ax = axes['expl_ynet_%d' % i]
        sigma = lambda x: 1. / (1. + np.exp(-x))
        y = sigma(np.dot(Z[tidx],V) + V0).mean(0).reshape(dims)
        vmin,vmax = explain_param['back_vmin'], explain_param['back_vmax']
        ax.imshow(y, vmin=vmin, vmax=vmax, cmap=pl.cm.gray_r, origin='lower')
        # True input
        ax = axes['expl_ydata_%d' % i]
        y = Y[tidx].mean(0).reshape(dims)
        ax.imshow(y, vmin=0, vmax=1, cmap=gc.input_cmap, origin='lower')


# # # # # # # # # # # # # #
# # # F I N A L I Z E # # #
# # # # # # # # # # # # # #


if SAVE is True:
    print " > Save figure to 'fig/Figure.{png,pdf}'."
    fig.savefig("fig/Figure.png")
    fig.savefig("fig/Figure.pdf")

print " > Show figure."
pl.show()
print " > Done."
