import numpy as np
import pylab as pl


def get_ax_print_size(ax):
    bbox = ax.get_window_extent().transformed(pl.gcf().dpi_scale_trans.inverted())
    width, height = bbox.width, bbox.height
    d = dict()
    d['inch'] = (width, height)
    width *= pl.rcParams['savefig.dpi']
    height *= pl.rcParams['savefig.dpi']
    d['pixel'] = (width, height)
    return d



def generate_DSL():
    from sys import argv
    if 'ipython' in argv[0]:
        maxlen = 4
    else:
        maxlen = 3
    if len(argv) == maxlen:     # DSL set manually
        DSL = argv[-1]
    else:
        import time
        DSL = '-'.join(("%02d" % i for i in time.localtime()[0:6]))
    return DSL


def extract_params(param, globals_dict, keep=(), first_level=True):
    """
    Bring all params in param into the current name space. 
    Provide the object dict as globals_dict; e.g. via globals()
    Stop dict-nesting for keys in keep.
    """
    if first_level: print "\n\t > Adding params to namespace:",
    for key, value in param.iteritems():
        if isinstance(value, dict) and (key not in keep):
            extract_params(value, globals_dict, keep, first_level=False)
        else:
            print key, ' ',
            globals_dict[key] = value
    if first_level: print '\n'



# STACKED BAR PLOT
def plot_stacked_bar(vals, centers=None, colors=None, width=0.8, labels=None, legend=False, ax=None, **Kwargs):
    '''
    Plots a stacked bar plot in axes ax.
    vals:    Values organized as vals[i,j] = height of the jth element in the ith bar.
    centers: x-centers of the bars; defaults to range(len(vals))
    colors:  colors[j] = face-color of the jth element; default colors exist.
    width:   width of the bars; defaults to 0.8
    labels:  Labels for the j-elements; defaults to ('1','2',...)
    legend:  boolean if to add a legend; defaults to False
    ax:      axes to draw into. If None: make a new figure.
    Kwargs:  params passed to matplotlib.bar (e.g. linewidth)

    Return: (axes, list of drawn instances, legend object if existing )
    '''
    if ax is None:
        fig = pl.figure(figsize=(8,5))
        ax = fig.add_axes((0.1,0.1,0.85,0.85))
    K,N = vals.shape    # K bars with N elements each
    if centers is None:
        centers = np.arange(K)
    if colors is None:
        colors = ('b', 'g', 'r', '0.3', 'c', '0.6', 'm', 'w') * 10

    drawnobj = []
    for n in xrange(N):
        if n > 0: bot = vals[:,:n].sum(1)
        else: bot = 0.
        c = colors[n % len(colors)]
        x = ax.bar(left=centers-width/2., height=vals[:,n], width=width, bottom=bot, color=c, **Kwargs)
        drawnobj.append(x)

    if legend:
        r = np.array(drawnobj)[::-1,0]
        if labels is None: labels = np.arange(1,N+1)
        labels = [labels[N-n] for n in xrange(1,N+1)]
        leg = ax.legend(r, labels, loc='upper right')
    else:
        leg = None


    ax.set_xlim(min(centers)-1.5*width/2., max(centers)+1.5*width/2.)
    ax.set_ylim(0., 1.05 * vals.sum(1).max())
    pl.draw()
    return ax, drawnobj, leg


