import numpy as np
import pylab as pl

def draw_scale(t_max, t_min, t_scale, bgcolor, ax):
    if t_scale is None:
        t_scales = np.array([0.010, 0.020, 0.050, 0.100, 0.200, 0.500, 1.000])
        t_scale = t_scales[abs((t_max - t_min)/8. - t_scales).argmin()]
    x = t_max - 1.0 * t_scale - 0.09 * (t_max - t_min)
    y = 0.02 * (ax.get_ylim()[1] - ax.get_ylim()[0])
    h = 0.12 * (ax.get_ylim()[1] - ax.get_ylim()[0])
    if isinstance(bgcolor, np.ndarray):
        bgcolor *= 0.90
    w = t_scale + 0.06 * (t_max - t_min)
    bgpatch = pl.Rectangle((x,y), w, h, lw=0., ec='w', fc=bgcolor, fill=True, alpha=0.7, zorder=2)
    ax.add_patch(bgpatch)
    x = t_max - t_scale - 0.06 * (t_max - t_min)
    y = 0.11 * (ax.get_ylim()[1] - ax.get_ylim()[0])
    h = 0.02 * (ax.get_ylim()[1] - ax.get_ylim()[0])
    patch = pl.Rectangle((x,y), t_scale, h, lw=0., ec='w', fc='k', fill=True, zorder=2)
    ax.add_patch(patch)
    x = t_max - 0.5 * t_scale - 0.06 * (t_max - t_min)
    y = 0.06 * (ax.get_ylim()[1] - ax.get_ylim()[0])
    l = "%d ms" % int(round(t_scale*1000))
    ax.text(x, y, l, ha='center', va='center', fontsize=6.)
    return


def plot_spiketrain(spikearray, t_min=0.0, t_max=1.0, dt=0.001, tau=None, ax=None):
    if ax is None:
        fig = pl.figure(figsize=(12,6))
        ax = fig.add_axes((0.1,0.1,0.85,0.85))
    K = spikearray.shape[-1]
    tn_min = int(round(t_min/dt))
    tn_max = int(round(t_max/dt))
    t,k = spikearray[tn_min:tn_max].nonzero()
    t = t * dt + t_min
    ms = 9. * 4./K
    ax.plot(t, k, 'k|', ms=ms, mew=0.5, aa=False)
    if tau is not None:
        tau = tau * dt
        for s,i in zip(t,k):
            p = pl.Rectangle((s,i-0.25), tau, 2*0.25, fc='0.8', lw=0., zorder=0)
            ax.add_patch(p)
    # ax.set_yticks(range(K))
    ax.set_xlim(t_min, t_max)
    ax.set_ylim(-0.5, K - 0.5)
    # ax.set_xlabel("Time [s]")
    # ax.set_ylabel("Neuron")
    pl.draw()
    return ax

def plot_styletrain(spikearray, t_min=0.0, t_max=1.0, dt=0.001, tau=None, ms=0.5, bgcolor='w', 
                    show_scale=True, t_scale=None, ax=None, **plotkwargs):
    if ax is None:
        fig = pl.figure(figsize=(12,6))
        ax = fig.add_axes((0.1,0.1,0.85,0.85))
    ax.set_axis_bgcolor(bgcolor)
    K = spikearray.shape[-1]
    tn_min = int(round(t_min/dt))
    tn_max = int(round(t_max/dt))
    t,k = spikearray[tn_min:tn_max].nonzero()
    t = t * dt + t_min
    # ms = 1. # 200. * 4./K
    defaultplotkwargs = dict(ms=ms, ls='None', marker='.', mec='k', mfc='k', aa=False, zorder=1)
    defaultplotkwargs.update(plotkwargs)
    ax.plot(t, k, **defaultplotkwargs)
    if tau is not None:
        tau = tau * dt
        for s,i in zip(t,k):
            p = pl.Rectangle((s,i-0.25), tau, 2*0.25, fc='0.8', lw=0., zorder=0)
            ax.add_patch(p)
    # ax.set_yticks(range(K))
    ax.set_xlim(t_min, t_max)
    ax.set_ylim(-0.5, K - 0.5)
    # add time scale
    if show_scale:
        draw_scale(t_max, t_min, t_scale, bgcolor, ax)
        #if t_scale is None:
            #t_scales = np.array([0.010, 0.020, 0.050, 0.100, 0.200, 0.500, 1.000])
            #t_scale = t_scales[abs((t_max - t_min)/8. - t_scales).argmin()]
        #x = t_max - 1.4 * t_scale - 0.1 * (t_max - t_min)
        #y = 0.02 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #h = 0.12 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #if isinstance(bgcolor, np.ndarray):
            #bgcolor *= 0.90
        #bgpatch = pl.Rectangle((x,y), 1.4*t_scale, h, lw=0., ec='w', fc=bgcolor, fill=True, alpha=0.7, zorder=2)
        #ax.add_patch(bgpatch)
        #x = t_max - 1.2 * t_scale - 0.1 * (t_max - t_min)
        #y = 0.11 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #h = 0.02 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #patch = pl.Rectangle((x,y), t_scale, h, lw=0., ec='w', fc='k', fill=True, zorder=2)
        #ax.add_patch(patch)
        #x = t_max - 0.7 * t_scale - 0.1 * (t_max - t_min)
        #y = 0.06 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #l = "%d ms" % int(round(t_scale*1000))
        #ax.text(x, y, l, ha='center', va='center', fontsize=6.)
    # Make axes "nice"
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    ax.set_xticks([])
    ax.set_yticks([0,K-1])
    ax.set_yticklabels([1,K])
    ax.yaxis.set_tick_params(size=0)
    #pl.draw()
    return ax



def plot_colored_styletrain(spikearray, t_min=0.0, t_max=1.0, dt=0.001, 
                            ctemplate=None, cidx=None, color=None, default_color='k', alpha=1.,
                            tau=None, ms=0.5, marker='.', bgcolor='w', show_scale=True,
                            t_scale=None, ax=None, **plot_kwargs):
    pl.interactive(False)
    if ax is None:
        fig = pl.figure(figsize=(12,6))
        ax = fig.add_axes((0.1,0.1,0.85,0.85))
    ax.set_axis_bgcolor(bgcolor)
    K = spikearray.shape[-1]
    if ctemplate is not None:
        assert ctemplate.shape[1] == cidx.shape[0]
        assert ctemplate.shape[1] == len(color)
        assert cidx.ndim == 2
    tn_min = int(round(t_min/dt))
    tn_max = int(round(t_max/dt))
    t,k = spikearray[tn_min:tn_max].nonzero()
    tn = t + tn_min
    t = t * dt + t_min
    # plot spikes
    spikes_by_color = dict.fromkeys(color)
    for c in color: spikes_by_color[c] = []
    spikes_by_color[default_color] = []
    for ti,tni,ki in zip(t, tn, k):
        c = default_color
        for p in ctemplate[tni].nonzero()[0]:
            if ki in cidx[p]:
                c = color[p]
                break
        spikes_by_color[c].append([ti,ki])
    for c in spikes_by_color:
        #return spikes_by_color[c]
        if len(spikes_by_color[c]) == 0:
            continue
        t,k = np.array(spikes_by_color[c]).T
        ax.plot(t, k, marker=marker, c=c, ms=ms, ls='None', aa=True, alpha=alpha,
                zorder=1, **plot_kwargs)
    #for ti,tni,ki in zip(t, tn, k):
        #c = default_color
        #for p in ctemplate[tni].nonzero()[0]:
            #if ki in cidx[p]:
                #c = color[p]
                #break
        #ax.plot(ti, ki, marker=marker, c=c, ms=ms, aa=True, zorder=1)
    if tau is not None:
        tau = tau * dt
        for s,i in zip(t,k):
            p = pl.Rectangle((s,i-0.25), tau, 2*0.25, fc='0.8', lw=0., zorder=0)
            ax.add_patch(p)
    ax.set_xlim(t_min, t_max)
    ax.set_ylim(-0.5, K - 0.5)
    # add time scale
    if show_scale:
        draw_scale(t_max, t_min, t_scale, bgcolor, ax)
        #if t_scale is None:
            #t_scales = np.array([0.010, 0.020, 0.050, 0.100, 0.200, 0.500, 1.000])
            #t_scale = t_scales[abs((t_max - t_min)/7. - t_scales).argmin()]
        #x = t_max - 1.7 * t_scale
        #y = 0.02 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #h = 0.12 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #if isinstance(bgcolor, np.ndarray):
            #bgcolor *= 0.90
        #bgpatch = pl.Rectangle((x,y), 1.4*t_scale, h, lw=0., ec='w', fc=bgcolor, fill=True, alpha=0.7, zorder=2)
        #ax.add_patch(bgpatch)
        #x = t_max - 1.5 * t_scale
        #y = 0.11 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #h = 0.02 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #patch = pl.Rectangle((x,y), t_scale, h, lw=0., ec='w', fc='k', fill=True, zorder=2)
        #ax.add_patch(patch)
        #x = t_max - 1.0 * t_scale
        #y = 0.06 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        #l = "%d ms" % int(round(t_scale*1000))
        #ax.text(x, y, l, ha='center', va='center', fontsize=8.)
    # Make axes "nice"
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    ax.set_xticks([])
    ax.set_yticks([0,K-1])
    ax.set_yticklabels([1,K])
    ax.yaxis.set_tick_params(size=0)
    pl.draw()
    pl.interactive(True)
    return ax



# # # # # # # # # # # # # # # # # # #
# # #  L I V E   P L O T T E R  # # #
# # # # # # # # # # # # # # # # # # #


class LiveRecurrent(object):
    def __init__(self, W, W_mask, dims, maxconn=50, seed=0, ax=None):
        if ax is None:
            self.fig = pl.figure()
            ax = self.fig.add_axes((0.1,0.1,0.8,0.8), aspect='equal')
        self.ax = ax
        self.W = W
        self.W_mask = W_mask
        self.maxconn = maxconn
        self.seed = seed
        self.dims = dims

    def update(self):
        # init
        pl.interactive(False)
        ax = self.ax
        dims = self.dims
        K = np.prod(dims)
        ax.clear()
        # add grid
        Y,X = np.meshgrid(np.arange(dims[0]), np.arange(dims[1]))
        ax.plot(X.flatten(), Y.flatten(), '.', ms=10, c='0.3')
        # get connection indices
        ks,js = (self.W_mask * np.tri(K,k=-1)).nonzero()
        idx = np.arange(len(ks))
        rng_state = np.random.get_state()
        np.random.seed(self.seed)
        np.random.shuffle(idx)
        np.random.set_state(rng_state)
        idx = idx[:self.maxconn]
        ks, js = ks[idx], js[idx]
        # plotting params
        wmax, wmin = self.W[self.W_mask].max(), self.W[self.W_mask].min()
        lw = lambda w: (5. - 0.5) * (w - wmin) / (wmax - wmin) + 0.5
        lc = lambda w: np.array([0.5,0.5,0.5]) + np.array([1.,-1.,-1.]) * (0.5 * (w - wmin) / (wmax - wmin))
        # plot
        for k,j in zip(ks,js):
            w = self.W[k,j]
            y,x = np.unravel_index([k,j], dims)
            ax.plot(x, y, ls='-', c=lc(w), lw=lw(w))
        # finalize
        ax.set_xticks(np.arange(0,dims[1],2))
        ax.set_yticks(np.arange(0,dims[0],2))
        ax.set_xlim(-0.5,dims[1]-0.5)
        ax.set_ylim(-0.5,dims[0]-0.5)
        ax.set_title("Wmax = %.2f" % wmax)
        pl.interactive(True)
        # make figure 'active'
        pl.figure(ax.get_figure().number)
        pl.draw()
        pl.draw()





class LivePlotter(object):
    def __init__(self, globalDict, plotW=True):
        pl.interactive(False)
        self.plotW = plotW
        self.linkalpha = lambda w: np.clip(abs(w) / 1., 0, 1)
        self.fig = pl.figure(figsize=(14,10))
        self.axes = dict()
        self.globaldict = globalDict
        # import global namespace
        globals().update(self.globaldict)
        # title
        rect = (0.01, 0.95, 0.98, 0.04)
        ax = self.fig.add_axes(rect, aspect='auto', frameon=False, xticks=[], yticks=[])
        self.axes['title'] = ax
        # recfield
        if self.plotW: rect = (0.01, 0.35, 0.59, 0.60)
        else: rect = (0.01, 0.35, 0.98, 0.60)
        ax = self.fig.add_axes(rect, aspect='equal') # equal
        ax.set_xticks(np.arange(-0.5, rdimsz[1]*rdimsy[1], rdimsy[1]))
        ax.set_yticks(np.arange(-0.5, rdimsz[0]*rdimsy[0], rdimsy[0]))
        ax.set_xticklabels([])
        ax.set_yticklabels([])
        ax.set_xlim(-0.5, rdimsz[1]*rdimsy[1]-0.5)
        ax.set_ylim(-0.5, rdimsz[0]*rdimsy[0]-0.5)
        ax.grid(1, linestyle='-')
        self.axes['recfield'] = ax
        # W matrix
        if self.plotW:
            rect = (0.65, 0.60, 0.30, 0.35)
            ax = self.fig.add_axes(rect, aspect='equal') # equal
            ax.set_xlim(-0.5, rdimsz[1]-0.5)
            ax.set_ylim(-0.5, rdimsz[0]-0.5)
            self.axes['W'] = ax
        # excitability
        rect = (0.65, 0.20, 0.30, 0.35)
        ax = self.fig.add_axes(rect, aspect='equal') # equal
        ax.set_xlim(-0.5, rdimsz[1]-0.5)
        ax.set_ylim(-0.5, rdimsz[0]-0.5)
        self.axes['b'] = ax
        # hist V
        rect = (0.05, 0.20, 0.55, 0.10)
        ax = self.fig.add_axes(rect, aspect='auto')
        self.axes['Vhist'] = ax
        # avg activity
        rect = (0.05, 0.05, 0.90, 0.10)
        ax = self.fig.add_axes(rect, aspect='auto')
        self.axes['zmean'] = ax
        pl.draw()
        pl.show()
        pl.interactive(True)

    def update(self):
        pl.interactive(False)
        titlesize = pl.rcParams["axes.titlesize"]
        pl.rc("axes", titlesize=10)
        # import global namespace
        globals().update(self.globaldict)
        # title
        ax = self.axes['title']
        ax.clear()
        s = "Simulation " + DSL + "      Time: %d (%.3fs)" % (t, t*dt)
        ax.text(0.5, 0.8, s, ha='center', va='center', weight='bold', fontsize=12)
        ax.set_xticks([])
        ax.set_yticks([])
        # recfield
        ax = self.axes['recfield']
        ax.clear()
        centers = V.get_centers()
        cax = ax.imshow( centers.reshape(rdimsz+rdimsy).swapaxes(1,2).reshape(\
                (rdimsz[0]*rdimsy[0], rdimsz[1]*rdimsy[1])), interpolation='nearest', cmap=pl.cm.gray_r)
        ax.set_title("Synaptic cluster centers (min=%.3f, max=%.3f)" % (centers.min(), centers.max()))
        ax.set_xticks(np.arange(-0.5, rdimsz[1]*rdimsy[1], rdimsy[1]))
        ax.set_yticks(np.arange(-0.5, rdimsz[0]*rdimsy[0], rdimsy[0]))
        ax.set_xticklabels([])
        ax.set_yticklabels([])
        ax.set_xlim(-0.5, rdimsz[1]*rdimsy[1]-0.5)
        ax.set_ylim(-0.5, rdimsz[0]*rdimsy[0]-0.5)
        ax.grid(1, linestyle='-')
        # W matrix
        if self.plotW:
            ax = self.axes['W']
            ax.clear()
            realW = (kwargs_connW['W_inh'] < W) * (W != 0)
            pairs = np.array(realW.nonzero())
            if pairs.size > 0:
                idx = np.arange(pairs.shape[1])
                np.random.shuffle(idx)
                pairs = pairs[:,idx[:100]]         # limit number of connections drawn
                xvals = []
                yvals = []
                colorcycle = []
                alphas = []
                for k,l in pairs.T:
                    if k > l: continue
                    #(y1,x1),(y2,x2) = net.pos_zk[k], net.pos_zk[l]
                    (y1,y2),(x1,x2) = np.unravel_index((k,l), rdimsz)
                    xvals.append([x1,x2])
                    yvals.append([y1,y2])
                    colorcycle.append((W[k,l] > 0) and 'r' or 'b')
                    alphas.append(np.clip(abs(W[k,l]),0,1))
                ax.set_color_cycle(colorcycle)
                ax.plot(np.transpose(xvals), np.transpose(yvals), lw=2.)
                for a,l in zip(alphas,ax.lines):
                    l.set_alpha(a)
                ax.set_xticks(np.arange(-0.5, rdimsz[1]-0.5, 1))
                ax.set_yticks(np.arange(-0.5, rdimsz[0]-0.5, 1))
                ax.set_xlim(-0.5, rdimsz[1]-0.5)
                ax.set_ylim(-0.5, rdimsz[0]-0.5)
                ax.set_xticklabels([])
                ax.set_yticklabels([])
                ax.grid(1, linestyle='-')
                ax.set_title("Recurrent weights w/o inhibition (min=%.3f, max=%.3f)" % (W[realW].min(), W[realW].max()))
        # excitability
        ax = self.axes['b']
        ax.clear()
        ax.imshow(b.reshape(rdimsz), interpolation='nearest', cmap=pl.cm.Reds, origin='lower')
        ax.set_xlim(-0.5, rdimsz[1]-0.5)
        ax.set_ylim(-0.5, rdimsz[0]-0.5)
        ax.set_xticks([])
        ax.set_yticks([])
        ax.set_title("Biases b (min=%.1f, max=%.1f)" % (b.min(), b.max()))
        # hist V
        ax = self.axes['Vhist']
        ax.clear()
        n,bins,patches = ax.hist( V[V_mask], bins=50, fc='0.8' )
        ax.set_xlim(np.min(bins), np.max(bins))
        ax.set_ylim(0, 1.05*np.max(n))
        ax.set_title("Input weight histogram (<V> = %.2f)" % ( V[V_mask].mean() ) )
        # avg activity
        ax = self.axes['zmean']
        ax.clear()
        if t > 0:
            ax.bar(np.arange(K), Z[t-PLOTEVERY:t].mean(0).mean(0), align='center')
        if isinstance(m_hom, float):
            hom = [m_hom]*K
        else:
            hom = m_hom
        ax.plot(np.transpose([(k-0.4,k+0.4) for k in xrange(K)]),
                np.transpose([(hom[k], hom[k]) for k in xrange(K)]), 'r-', lw=1.)
        ax.set_xlim(-1,K)
        ax.set_title("Average z activity based on last %dx%d samples" % (PLOTEVERY, num_parallel))
        # make figure 'active'
        pl.figure(ax.get_figure().number)
        pl.draw()
        pl.draw()
        pl.rc("axes", titlesize=titlesize)
        pl.interactive(True)



