import numpy as np

def covariance_plasticity_tan(W, z, m, Wmax, gamma):
    """
    dW = x*y - [ m^2 + 1/gamma * tan(pi/2 * W/Wmax)
      --> W = Wmax/(pi/2) * arctan(gamma * Cov(x,y))
      with homeostatic target m.
    """
    num_parallel = z.shape[0]
    factor = np.pi / 2. / Wmax
    dW = np.dot(z.T,z) - num_parallel * (m**2 + (1./gamma) * np.tan(factor * W))
    dW -= np.diag(dW.diagonal())      # neglect self-connections
    return dW



def wake_sleep_plasticity(z, z_prior):
    """
    dW[k,j] = <z[k]*z[j]> - <z_prior[k]*z_prior[j]>
    db_prior[k] = <z[k]> - <z_prior[k]>

    returns: db_prior, dW
    """
    npost = z.shape[0]
    npri = z_prior.shape[0]
    q = float(npost) / npri
    dW = np.dot(z.T,z) - q * np.dot(z_prior.T,z_prior)
    db_prior = dW.diagonal()
    dW -= np.diag(db_prior)
    return db_prior, dW

