import numpy as np
import pylab as pl
import shared
from panel import EmptyPanel
from figure import build_fig

class LivePlot(object):
    """ Core Class """
    def __init__(self, sim_globals):
        # load panels and figure layout
        from lpconf import FIGSIZE, DPI, LAYOUT, panels
        pl.rc('figure', dpi=DPI)
        self.fig, self.axes = build_fig(FIGSIZE, LAYOUT)
        self.fig.canvas.manager.set_window_title("LivePlot")
        # remove non-displayed panels
        active = np.unique(np.array(LAYOUT, dtype=int))
        names = panels.keys()
        for name in names:
            if name not in active:
                panels.pop(name)
        self.paneldict = panels
        self.panel = dict()
        # generate panels and connect to axes
        for name, ax in self.axes.iteritems():
            panelClass = self.paneldict[name]['panelClass']
            self.panel[name] = panelClass(ax=ax)
        # init status bar
        try:
            from lpconf import status
            self.statusdict = status
        except:
            self.statusdict = None
        self.fig.canvas.manager.set_window_title("LivePlot")
        self._attach(sim_globals)
        self._running = False

    def _attach(self, sim_globals):
        # collect attach_keys
        attach_keys = []
        for p in self.paneldict.itervalues():
            if p['panelClass'] is EmptyPanel:
                continue
            for key in p['attach']:
                if key not in attach_keys:
                    attach_keys.append(key)
        if self.statusdict is not None:
            for key in self.statusdict['attach']:
                if key not in attach_keys:
                    attach_keys.append(key)
        # collect dependent functions
        dependent_func = []
        for p in self.paneldict.itervalues():
            if p['panelClass'] is EmptyPanel:
                continue
            dependent_func.append(p['initKwargs'])
            dependent_func.append(p['updateKwargs'])
        if self.statusdict is not None:
            dependent_func.append(self.statusdict['text'])
        # register at shared
        shared.attach(sim_globals, attach_keys, dependent_func)

    def _start(self):
        shared.update()
        for name,p in self.paneldict.iteritems():
            if isinstance(self.panel[name], EmptyPanel):
                continue
            self.panel[name].setup(**p['initKwargs']())
        pl.figure(self.fig.number)
        pl.draw()
        pl.show()
        self._running = True

    def update(self):
        pl.interactive(False)
        if self._running is False:
            self._start()
        shared.update()
        for name,p in self.paneldict.iteritems():
            if isinstance(self.panel[name], EmptyPanel):
                continue
            self.panel[name].update(**p['updateKwargs']())
        if self.statusdict is not None:
            s = "LivePlot :  "
            s += self.statusdict['text']()
            self.fig.canvas.manager.set_window_title(s)
        pl.figure(self.fig.number)
        pl.draw()
        pl.draw()
        pl.interactive(True)
