import numpy as np
import pylab as pl
from BaseClassPanel import BaseClassPanel

class BarPanel(BaseClassPanel):
    def setup(self, xlabel='', ylabel='', target=None, log=False, targetcolor='r', barcolor='b'):
        self.xlabel = xlabel
        self.ylabel = ylabel
        from collections import Iterable
        self.iter_target = False
        if isinstance(target, Iterable):
            target = np.array(target)
            self.iter_target = True
        self.target = target
        self.log = log
        self.tc = targetcolor
        self.bc = barcolor

    def update(self, data, center=None, title=''):
        # check data types
        if not isinstance(data, np.ndarray):
            data = np.array(data)
        assert data.ndim == 1, "Data must be one dimensional!"
        if center is None:
            center = np.arange(len(data))
        if not isinstance(center, np.ndarray):
            center = np.array(center)
        assert center.shape == data.shape, "Centers do not fit data!"
        # plot
        ax = self.ax
        ax.clear()
        c = np.sort(center)
        w = 0.9 * (c[1:]-c[:-1]).min()
        ax.bar(center, data, width=w, align='center', log=self.log, color=self.bc, ec='0.1')
        dmax = data.max()
        if self.target is not None:
            tar = self.target
            if not self.iter_target:
                tar = np.array([tar] * len(center))
            assert len(center) == len(tar), "Target does not match data shape!"
            dmax = max(tar.max(), dmax)
            for x,y in zip(center, tar):
                ax.plot((x-0.35*w,x+0.35*w), (y,y), color=self.tc, lw=1.5)
        ax.set_title(title)
        ax.set_xlabel(self.xlabel)
        ax.set_ylabel(self.ylabel)
        ax.set_xlim(center.min()-0.7*w, center.max()+0.7*w)
        ymin = self.log and 0.5*data.min() or 0.
        ax.set_ylim(ymin, 1.1*dmax)


