import numpy as np
import pylab as pl
from BaseClassPanel import BaseClassPanel

class HistogramPanel(BaseClassPanel):
    def setup(self, xlabel='', ylabel='', trace=False, log=False, normed=False, barcolor='b'):
        self.xlabel = xlabel
        self.ylabel = ylabel
        self.trace = trace
        self.trace_data = None
        self.log = log
        self.normed = normed
        self.bc = barcolor

    def update(self, data, bins=20, title=''):
        # check data types
        if not isinstance(data, np.ndarray):
            data = np.array(data)
        assert data.ndim == 1, "Data must be one dimensional!"
        from collections import Iterable
        if isinstance(bins, Iterable):
            bins = bins
        elif isinstance(bins, int) and (self.trace is False):
            bins = np.linspace(data.min(), data.max(), bins+1)
        elif isinstance(bins, int) and (self.trace is True):
            dmin, dmax = data.min(), data.max()
            if self.trace_data is not None:
                dmin, dmax = min(dmin, self.trace_data.min()), max(dmax, self.trace_data.max())
            bins = np.linspace(dmin, dmax, bins+1)
        else:
            raise Exception, "bins must be int or sequence; trace must be True or False."
        # plot
        ax = self.ax
        ax.clear()
        alpha = self.trace and 0.8 or 1.0
        n,bins,patches = ax.hist(data, bins, rwidth=1.0, normed=self.normed, log=self.log, 
                                 fc=self.bc, ec='0.1', zorder=1, alpha=alpha)
        nmin, nmax = n[n>0].min(), n.max()
        if self.trace and (self.trace_data is not None):
            n,bins,patches = ax.hist(self.trace_data, bins, rwidth=0.8, normed=self.normed, 
                                     log=self.log, fc='0.9', ec='0.4', zorder=0)
            nmin, nmax = min(nmin, n[n>0].min()), max(nmax, n.max())
            pass
        if self.trace:
            self.trace_data = data.copy()
        ax.set_title(title)
        ax.set_xlabel(self.xlabel)
        ax.set_ylabel(self.ylabel)
        d = 0.03 * (bins[-1] - bins[0])
        ax.set_xlim(bins[0]-d, bins[-1]+d)
        ymin = self.log and 0.5*nmin or 0.
        ax.set_ylim(ymin, 1.1*nmax)


