import numpy as np
import pylab as pl
from BaseClassPanel import BaseClassPanel

class ImageGridPanel(BaseClassPanel):
    def setup(self, symmetric=False, vrange=(None,None), cmap=pl.cm.RdBu_r, interpolation='nearest',
                    aspect='equal', origin='lower', colorbar=True, cblabel=''):
        self.symmetric = symmetric
        self.vmin, self.vmax = vrange
        self.cmap = cmap
        self.interpolation = interpolation
        self.aspect = aspect
        self.origin = origin
        self.colorbar = colorbar
        self.cblabel = cblabel
        self._cb = None
        self.ax.set_aspect(self.aspect)

    def update(self, data, title=''):
        # check data types
        if not isinstance(data, np.ndarray):
            data = np.array(data)
        assert data.ndim == 2, "Data must be two dimensional!"
        ax = self.ax
        ax.clear()
        dims = data.shape
        # vrange
        vmin, vmax = self.vmin, self.vmax
        if vmin is None:
            vmin = data.min()
        if vmax is None:
            vmax = data.max()
        if self.symmetric:
            vmin, vmax = -max(abs(vmin), abs(vmax)), +max(abs(vmin), abs(vmax))
        # plot
        kwargs = dict(cmap=self.cmap, interpolation=self.interpolation, aspect=self.aspect, origin=self.origin,
                      vmin=vmin, vmax=vmax)
        mappable = ax.imshow(data, **kwargs)
        if self.colorbar:
            fig = ax.get_figure()
            if self._cb is None:
                self._cb = fig.colorbar(mappable, ax=ax, shrink=0.5)
            else:
                self._cb.ax.clear()
                self._cb = fig.colorbar(mappable, cax=self._cb.ax)
            self._cb.set_label(self.cblabel)
        ax.set_title(title)
        ax.set_xlabel('')
        ax.set_ylabel('')
        ax.set_xlim(-0.5, dims[1]-0.5)
        ax.set_ylim(-0.5, dims[0]-0.5)


