import numpy as np
import pylab as pl
from BaseClassPanel import BaseClassPanel

class ReceptiveFieldsPanel(BaseClassPanel):
    def setup(self, post_dims, rf_dims, V_mask, colorbar=True, cblabel=''):
        self.post_dims = post_dims
        self.rf_dims = rf_dims
        self.V_mask = V_mask
        self.colorbar = colorbar
        self.cblabel = cblabel
        self._cb = None
        self.ax.set_aspect('equal')

    def update(self, data, title=''):
        # check data types
        if not isinstance(data, np.ndarray):
            data = np.array(data)
        assert data.ndim == 2, "W must be two dimensional!"
        rfd = self.rf_dims
        zd = self.post_dims
        K = np.prod(zd)
        V_mask = self.V_mask
        assert data.shape[0] == K, "data does not match post_dims!"
        assert data.shape == V_mask.shape, "data does not match V_mask!"
        ax = self.ax
        ax.clear()
        # prepare data
        X = np.zeros((zd[0]*rfd[0], zd[1]*rfd[1]))
        for k,(i,j) in enumerate(np.ndindex(*zd)):
            X[rfd[0]*i:rfd[0]*(i+1), rfd[1]*j:rfd[1]*(j+1)] = data[k,V_mask[k]].reshape(rfd)
        kwargs = dict(cmap=pl.cm.bone_r, interpolation='nearest', aspect='equal', origin='lower')
        # plot
        mappable = ax.imshow(X, **kwargs)
        if self.colorbar:
            fig = ax.get_figure()
            if self._cb is None:
                self._cb = fig.colorbar(mappable, ax=ax, shrink=0.5)
            else:
                self._cb.ax.clear()
                self._cb = fig.colorbar(mappable, cax=self._cb.ax)
            self._cb.set_label(self.cblabel)
        # finalize
        dims = X.shape
        ax.set_xlabel('')
        ax.set_ylabel('')
        # major
        ax.set_xticks(np.arange(0,dims[1],rfd[1])-0.5)
        ax.set_yticks(np.arange(0,dims[0],rfd[0])-0.5)
        ax.grid(True, ls='-', lw=1.0, c='orangered')
        for child in ax.get_children():
            if isinstance(child, pl.matplotlib.spines.Spine):
                child.set_color('orangered')
                child.set_lw(1.)
        ax.set_xticklabels([])
        ax.set_yticklabels([])
        # minor
        ax.set_xticks(np.arange((rfd[1]-1)/2.,dims[1],rfd[1]), minor=True)
        ax.set_yticks(np.arange((rfd[0]-1)/2.,dims[0],rfd[0]), minor=True)
        ax.set_xticklabels(np.arange(zd[1]), minor=True)
        ax.set_yticklabels(np.arange(zd[0]), minor=True)
        # limits
        ax.set_xlim(-0.5,dims[1]-0.5)
        ax.set_ylim(-0.5,dims[0]-0.5)
        ax.set_title(title)


