import numpy as np
import pylab as pl
from BaseClassPanel import BaseClassPanel

class SpiketrainPanel(BaseClassPanel):
    def setup(self, t_min=0.0, t_max=1.0, dt=0.001, tau=None, **plotKwargs):
        self.t_min = t_min
        self.t_max = t_max
        self.dt = dt
        self.tau = tau
        self.plotKwargs = plotKwargs

    def update(self, data, title=''):
        # check data types
        if not isinstance(data, np.ndarray):
            data = np.array(data)
        assert data.ndim == 2, "Data must be two dimensional!"
        ax = self.ax
        ax.clear()
        dims = data.shape
        plot_styletrain(spikearray=data, t_min=self.t_min, t_max=self.t_max, dt=self.dt, tau=self.tau,
                        bgcolor='w', show_scale=True, ax=ax, **self.plotKwargs)
        ax.set_title(title)




def plot_styletrain(spikearray, t_min=0.0, t_max=1.0, dt=0.001, tau=None, bgcolor='w', 
                    show_scale=True, ax=None, **plotkwargs):
    ax.set_axis_bgcolor(bgcolor)
    K = spikearray.shape[-1]
    tn_min = int(round(t_min/dt))
    tn_max = int(round(t_max/dt))
    t,k = spikearray[tn_min:tn_max].nonzero()
    t = t * dt + t_min
    # ms = 1. # 200. * 4./K
    defaultplotkwargs = dict(ms=1.0, mew=0.5, ls='None', marker='|', mec='k', mfc='k', aa=False, zorder=1)
    defaultplotkwargs.update(plotkwargs)
    ax.plot(t, k, **defaultplotkwargs)
    if tau is not None:
        tau = tau * dt
        for s,i in zip(t,k):
            p = pl.Rectangle((s,i-0.25), tau, 2*0.25, fc='0.8', lw=0., zorder=0)
            ax.add_patch(p)
    # ax.set_yticks(range(K))
    ax.set_xlim(t_min, t_max)
    ax.set_ylim(-0.5, K - 0.5)
    # add time scale
    if show_scale:
        t_scales = np.array([0.010, 0.020, 0.050, 0.100, 0.200, 0.500, 1.000])
        t_scale = t_scales[abs((t_max - t_min)/10. - t_scales).argmin()]
        x = t_max - 1.7 * t_scale
        y = 0.02 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        h = 0.12 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        if isinstance(bgcolor, np.ndarray):
            bgcolor *= 0.90
        bgpatch = pl.Rectangle((x,y), 1.4*t_scale, h, lw=0., ec='w', fc=bgcolor, fill=True, alpha=0.7, zorder=2)
        ax.add_patch(bgpatch)
        x = t_max - 1.5 * t_scale
        y = 0.11 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        h = 0.02 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        patch = pl.Rectangle((x,y), t_scale, h, lw=0., ec='w', fc='k', fill=True, zorder=2)
        ax.add_patch(patch)
        x = t_max - 1.0 * t_scale
        y = 0.06 * (ax.get_ylim()[1] - ax.get_ylim()[0])
        l = "%d ms" % int(round(t_scale*1000))
        ax.text(x, y, l, ha='center', va='center', fontsize=8.)
    # Make axis "nice"
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    ax.set_xticks([])
    ax.set_yticks([0,K-1])
    ax.set_yticklabels([0,K-1])
    ax.yaxis.set_tick_params(size=0)
