#from types import ModuleType

class SharedParam(object):
    def __init__(self, mod):
        self._saved_module = mod
        self._dict = dict()
        self._sim_globals = {}
        self._attach_keys = []
        self._dependent_func = []
        self._is_attached = False

    def __getattr__(self, x):
        if x in self._dict.keys():
            return self._dict[x]
        else:
            return getattr(self._saved_module, x)

    def __setattr__(self,key,val):
        if key in('_saved_module', '_dict', '_sim_globals', '_attach_keys', '_dependent_func', '_is_attached'):
            super(SharedParam, self).__setattr__(key, val)
        elif key in self._attach_keys:
            self._dict[key] = val
        else:
            raise AttributeError, "Shared parameter '%s' not registered!" % key

    def attach(self, sim_globals, attach_keys, dependent_func):
        assert self._is_attached is False, "ERROR: Shared params have already been attached!"
        self._sim_globals = sim_globals
        self._attach_keys = attach_keys
        self._dependent_func = dependent_func
        # Make sure attach_keys do not compromize dependent_func.func_globals
        conflict = []
        for key in self._attach_keys:
            for f in self._dependent_func:
                if (key in f.func_globals.keys()) and (key not in conflict):
                    conflict.append(key)
        if len(conflict) > 0:
            raise Exception, "ERROR: The following attached variables conflict with objects in the configuration file:\n %s" % str(conflict)
        # Do not yet update; wait for first drawing (e.g. param 't' may only exist later)
        self._is_attached = True

    def update(self):
        assert self._is_attached is True, "ERROR: Shared params have not been attached!"
        # fetch from globals
        for key in self._attach_keys:
            try:
                setattr(self, key, self._sim_globals[key])
            except KeyError:
                import sys
                e = sys.exc_info()[0]
                raise e("Is registered shared variable '%s' available in simulation?" % key)
            except Exception as e:
                raise e
        # distribute to functions
        for f in self._dependent_func:
            f.func_globals.update(self._dict)

import sys
sys.modules[__name__] = SharedParam(sys.modules[__name__])
